@@ -532,6 +532,7 @@ static int set_volume_mute(snd_ctl_pipewire_t *ctl, const char *name, struct vol
 		spa_pod_builder_add(&b,
 			SPA_PARAM_ROUTE_index, SPA_POD_Int(id),
 			SPA_PARAM_ROUTE_device, SPA_POD_Int(device_id),
+			SPA_PARAM_ROUTE_save, SPA_POD_Bool(true),
 			0);
 
 		spa_pod_builder_prop(&b, SPA_PARAM_ROUTE_props, 0);
@@ -784,9 +785,9 @@ static const snd_ctl_ext_callback_t pipewire_ext_callback = {
 };
 
 /** device */
-static void device_event_info(void *object, const struct pw_device_info *info)
+static void device_event_info(void *data, const struct pw_device_info *info)
 {
-	struct global *g = object;
+	struct global *g = data;
 	snd_ctl_pipewire_t *ctl = g->ctl;
 	uint32_t n;
 
@@ -864,11 +865,11 @@ static struct global *find_node_for_route(snd_ctl_pipewire_t *ctl, uint32_t card
 	return NULL;
 }
 
-static void device_event_param(void *object, int seq,
+static void device_event_param(void *data, int seq,
                 uint32_t id, uint32_t index, uint32_t next,
                 const struct spa_pod *param)
 {
-	struct global *g = object;
+	struct global *g = data;
 	snd_ctl_pipewire_t *ctl = g->ctl;
 
 	pw_log_debug("param %d", id);
@@ -922,9 +923,9 @@ static const struct global_info device_info = {
 };
 
 /** node */
-static void node_event_info(void *object, const struct pw_node_info *info)
+static void node_event_info(void *data, const struct pw_node_info *info)
 {
-	struct global *g = object;
+	struct global *g = data;
 	snd_ctl_pipewire_t *ctl = g->ctl;
 	const char *str;
 	uint32_t i;
@@ -970,11 +971,11 @@ static void node_event_info(void *object, const struct pw_node_info *info)
 }
 
 
-static void node_event_param(void *object, int seq,
+static void node_event_param(void *data, int seq,
                 uint32_t id, uint32_t index, uint32_t next,
                 const struct spa_pod *param)
 {
-	struct global *g = object;
+	struct global *g = data;
 	pw_log_debug("update param %d %d", g->id, id);
 
 	switch (id) {
@@ -1010,7 +1011,7 @@ static int json_object_find(const char *obj, const char *key, char *value, size_
 	if (spa_json_enter_object(&it[0], &it[1]) <= 0)
 		return -EINVAL;
 
-	while (spa_json_get_string(&it[1], k, sizeof(k)-1) > 0) {
+	while (spa_json_get_string(&it[1], k, sizeof(k)) > 0) {
 		if (spa_streq(k, key)) {
 			if (spa_json_get_string(&it[1], value, len) <= 0)
 				continue;
@@ -1023,13 +1024,13 @@ static int json_object_find(const char *obj, const char *key, char *value, size_
 	return -ENOENT;
 }
 
-static int metadata_property(void *object,
+static int metadata_property(void *data,
                         uint32_t subject,
                         const char *key,
                         const char *type,
                         const char *value)
 {
-	struct global *g = object;
+	struct global *g = data;
 	snd_ctl_pipewire_t *ctl = g->ctl;
 
 	if (subject == PW_ID_CORE) {
@@ -1082,6 +1083,7 @@ static void proxy_destroy(void *data)
 	struct global *g = data;
 	spa_list_remove(&g->link);
 	g->proxy = NULL;
+	pw_properties_free(g->props);
 }
 
 static const struct pw_proxy_events proxy_events = {
@@ -1165,15 +1167,19 @@ static void registry_event_global_remove(void *data, uint32_t id)
 	struct global *g;
 	const char *name;
 
-	if ((g = find_global(ctl, id, NULL, PW_TYPE_INTERFACE_Node)) == NULL)
-		return;
-	if ((name = pw_properties_get(g->props, PW_KEY_NODE_NAME)) == NULL)
+	if ((g = find_global(ctl, id, NULL, NULL)) == NULL)
 		return;
 
-	if (spa_streq(name, ctl->default_sink))
-		ctl->default_sink[0] = '\0';
-	if (spa_streq(name, ctl->default_source))
-		ctl->default_source[0] = '\0';
+	if (spa_streq(g->ginfo->type, PW_TYPE_INTERFACE_Node)) {
+		if ((name = pw_properties_get(g->props, PW_KEY_NODE_NAME)) == NULL)
+			return;
+
+		if (spa_streq(name, ctl->default_sink))
+			ctl->default_sink[0] = '\0';
+		if (spa_streq(name, ctl->default_source))
+			ctl->default_source[0] = '\0';
+	}
+	pw_proxy_destroy(g->proxy);
 }
 
 static const struct pw_registry_events registry_events = {
