@@ -31,7 +31,6 @@
 
 #include <alsa/asoundlib.h>
 
-#include <spa/support/log.h>
 #include <spa/utils/type.h>
 #include <spa/node/node.h>
 #include <spa/utils/keys.h>
@@ -48,12 +47,12 @@
 #include <spa/pod/parser.h>
 #include <spa/debug/pod.h>
 
+#include "alsa.h"
+
 #include "acp/acp.h"
 
 extern struct spa_i18n *acp_i18n;
 
-#define NAME  "alsa-device"
-
 #define MAX_POLL	16
 
 #define DEFAULT_DEVICE		"hw:0"
@@ -217,7 +216,7 @@ static int emit_info(struct impl *this, bool full)
 {
 	int err = 0;
 	struct spa_dict_item *items;
-	uint32_t i, n_items;
+	uint32_t n_items;
 	const struct acp_dict_item *it;
 	struct acp_card *card = this->card;
 	char path[128];
@@ -242,10 +241,10 @@ static int emit_info(struct impl *this, bool full)
 #undef ADD_ITEM
 
 		if (this->info.change_mask & SPA_DEVICE_CHANGE_MASK_PARAMS) {
-			for (i = 0; i < SPA_N_ELEMENTS(this->params); i++) {
-				if (this->params[i].user > 0) {
-					this->params[i].flags ^= SPA_PARAM_INFO_SERIAL;
-					this->params[i].user = 0;
+			SPA_FOR_EACH_ELEMENT_VAR(this->params, p) {
+				if (p->user > 0) {
+					p->flags ^= SPA_PARAM_INFO_SERIAL;
+					p->user = 0;
 				}
 			}
 		}
@@ -1043,6 +1042,8 @@ impl_init(const struct spa_handle_factory *factory,
 	this = (struct impl *) handle;
 
 	this->log = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_Log);
+	alsa_log_topic_init(this->log);
+
 	this->loop = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_Loop);
 	acp_i18n = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_I18N);
 	if (this->loop == NULL) {
