@@ -56,9 +56,9 @@ static void emit_node_info(struct state *this, bool full)
 	if (full)
 		this->info.change_mask = this->info_all;
 	if (this->info.change_mask) {
-		struct spa_dict_item items[4];
+		struct spa_dict_item items[7];
 		uint32_t i, n_items = 0;
-		char latency[64];
+		char latency[64], period[64], nperiods[64], headroom[64];
 
 		items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_DEVICE_API, "alsa");
 		items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_MEDIA_CLASS, "Audio/Sink");
@@ -66,6 +66,12 @@ static void emit_node_info(struct state *this, bool full)
 		if (this->have_format) {
 			snprintf(latency, sizeof(latency), "%lu/%d", this->buffer_frames / 2, this->rate);
 			items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_NODE_MAX_LATENCY, latency);
+			snprintf(period, sizeof(period), "%lu", this->period_frames);
+			items[n_items++] = SPA_DICT_ITEM_INIT("api.alsa.period-size", period);
+			snprintf(nperiods, sizeof(nperiods), "%lu", this->buffer_frames / this->period_frames);
+			items[n_items++] = SPA_DICT_ITEM_INIT("api.alsa.period-num", nperiods);
+			snprintf(headroom, sizeof(headroom), "%u", this->headroom);
+			items[n_items++] = SPA_DICT_ITEM_INIT("api.alsa.headroom", headroom);
 		}
 		this->info.props = &SPA_DICT_INIT(items, n_items);
 
@@ -707,6 +713,7 @@ impl_node_port_set_param(void *object,
 		this->port_info.change_mask |= SPA_PORT_CHANGE_MASK_PARAMS;
 		this->port_params[PORT_Latency].user++;
 		emit_port_info(this, false);
+		res = 0;
 		break;
 	}
 	default:
@@ -724,6 +731,7 @@ impl_node_port_use_buffers(void *object,
 {
 	struct state *this = object;
 	uint32_t i;
+	int res;
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 
@@ -731,14 +739,15 @@ impl_node_port_use_buffers(void *object,
 
 	spa_log_debug(this->log, "%p: use %d buffers", this, n_buffers);
 
-	if (!this->have_format)
-		return -EIO;
-
-	if (n_buffers == 0) {
+	if (this->n_buffers > 0) {
 		spa_alsa_pause(this);
-		clear_buffers(this);
-		return 0;
+		if ((res = clear_buffers(this)) < 0)
+			return res;
 	}
+	if (n_buffers > 0 && !this->have_format)
+		return -EIO;
+	if (n_buffers > MAX_BUFFERS)
+		return -ENOSPC;
 
 	for (i = 0; i < n_buffers; i++) {
 		struct buffer *b = &this->buffers[i];
@@ -797,38 +806,38 @@ static int impl_node_port_reuse_buffer(void *object, uint32_t port_id, uint32_t
 static int impl_node_process(void *object)
 {
 	struct state *this = object;
-	struct spa_io_buffers *input;
+	struct spa_io_buffers *io;
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 
-	input = this->io;
-	spa_return_val_if_fail(input != NULL, -EIO);
+	if ((io = this->io) == NULL)
+		return -EIO;
 
-	spa_log_trace_fp(this->log, "%p: process %d %d/%d", this, input->status,
-			input->buffer_id, this->n_buffers);
+	spa_log_trace_fp(this->log, "%p: process %d %d/%d", this, io->status,
+			io->buffer_id, this->n_buffers);
 
 	if (this->position && this->position->clock.flags & SPA_IO_CLOCK_FLAG_FREEWHEEL) {
-		input->status = SPA_STATUS_NEED_DATA;
+		io->status = SPA_STATUS_NEED_DATA;
 		return SPA_STATUS_HAVE_DATA;
 	}
-	if (input->status == SPA_STATUS_HAVE_DATA &&
-	    input->buffer_id < this->n_buffers) {
-		struct buffer *b = &this->buffers[input->buffer_id];
+	if (io->status == SPA_STATUS_HAVE_DATA &&
+	    io->buffer_id < this->n_buffers) {
+		struct buffer *b = &this->buffers[io->buffer_id];
 
 		if (!SPA_FLAG_IS_SET(b->flags, BUFFER_FLAG_OUT)) {
 			spa_log_warn(this->log, "%p: buffer %u in use",
-					this, input->buffer_id);
-			input->status = -EINVAL;
+					this, io->buffer_id);
+			io->status = -EINVAL;
 			return -EINVAL;
 		}
-		spa_log_trace_fp(this->log, "%p: queue buffer %u", this, input->buffer_id);
+		spa_log_trace_fp(this->log, "%p: queue buffer %u", this, io->buffer_id);
 		spa_list_append(&this->ready, &b->link);
 		SPA_FLAG_CLEAR(b->flags, BUFFER_FLAG_OUT);
-		input->buffer_id = SPA_ID_INVALID;
+		io->buffer_id = SPA_ID_INVALID;
 
 		spa_alsa_write(this);
 
-		input->status = SPA_STATUS_OK;
+		io->status = SPA_STATUS_OK;
 	}
 	return SPA_STATUS_HAVE_DATA;
 }
