@@ -25,9 +25,11 @@
 #include <stdio.h>
 
 #include <gst/gst.h>
+#include <gst/allocators/gstdmabuf.h>
 #include <gst/video/video.h>
 #include <gst/audio/audio.h>
 
+#include <spa/utils/string.h>
 #include <spa/utils/type.h>
 #include <spa/param/video/format-utils.h>
 #include <spa/param/audio/format-utils.h>
@@ -35,6 +37,14 @@
 
 #include "gstpipewireformat.h"
 
+#ifndef DRM_FORMAT_MOD_INVALID
+#define DRM_FORMAT_MOD_INVALID ((1ULL << 56) - 1)
+#endif
+
+#ifndef DRM_FORMAT_MOD_LINEAR
+#define DRM_FORMAT_MOD_LINEAR 0
+#endif
+
 struct media_type {
   const char *name;
   uint32_t media_type;
@@ -47,6 +57,8 @@ static const struct media_type media_type_map[] = {
   { "image/jpeg", SPA_MEDIA_TYPE_video, SPA_MEDIA_SUBTYPE_mjpg },
   { "video/x-jpeg", SPA_MEDIA_TYPE_video, SPA_MEDIA_SUBTYPE_mjpg },
   { "video/x-h264", SPA_MEDIA_TYPE_video, SPA_MEDIA_SUBTYPE_h264 },
+  { "audio/x-mulaw", SPA_MEDIA_TYPE_audio, SPA_MEDIA_SUBTYPE_raw },
+  { "audio/x-alaw", SPA_MEDIA_TYPE_audio, SPA_MEDIA_SUBTYPE_raw },
   { NULL, }
 };
 
@@ -188,7 +200,7 @@ find_media_types (const char *name)
 {
   int i;
   for (i = 0; media_type_map[i].name; i++) {
-    if (!strcmp (media_type_map[i].name, name))
+    if (spa_streq(media_type_map[i].name, name))
       return &media_type_map[i];
   }
   return NULL;
@@ -299,6 +311,13 @@ get_nth_rectangle (const GValue *width, const GValue *height, int idx, struct sp
       r->width = gst_value_get_int_range_max (width);
       r->height = gst_value_get_int_range_max (height);
       return true;
+    } else if (idx == 3) {
+      r->width = gst_value_get_int_range_step (width);
+      r->height = gst_value_get_int_range_step (height);
+      if (r->width > 1 || r->height > 1)
+        return true;
+      else
+        return false;
     }
   } else if (wt == GST_TYPE_LIST && ht == GST_TYPE_LIST) {
     GArray *wa = g_value_peek_pointer (width);
@@ -442,6 +461,44 @@ handle_video_fields (ConvertData *d)
   return TRUE;
 }
 
+static void
+set_default_channels (struct spa_pod_builder *b, uint32_t channels)
+{
+  uint32_t position[SPA_AUDIO_MAX_CHANNELS] = {0};
+  gboolean ok = TRUE;
+
+  switch (channels) {
+  case 8:
+    position[6] = SPA_AUDIO_CHANNEL_SL;
+    position[7] = SPA_AUDIO_CHANNEL_SR;
+    SPA_FALLTHROUGH
+  case 6:
+    position[5] = SPA_AUDIO_CHANNEL_LFE;
+    SPA_FALLTHROUGH
+  case 5:
+    position[4] = SPA_AUDIO_CHANNEL_FC;
+    SPA_FALLTHROUGH
+  case 4:
+    position[2] = SPA_AUDIO_CHANNEL_RL;
+    position[3] = SPA_AUDIO_CHANNEL_RR;
+    SPA_FALLTHROUGH
+  case 2:
+    position[0] = SPA_AUDIO_CHANNEL_FL;
+    position[1] = SPA_AUDIO_CHANNEL_FR;
+    break;
+  case 1:
+    position[0] = SPA_AUDIO_CHANNEL_MONO;
+    break;
+  default:
+    ok = FALSE;
+    break;
+  }
+
+  if (ok)
+    spa_pod_builder_add (b, SPA_FORMAT_AUDIO_position,
+        SPA_POD_Array(sizeof(uint32_t), SPA_TYPE_Id, channels, position), 0);
+}
+
 static gboolean
 handle_audio_fields (ConvertData *d)
 {
@@ -469,6 +526,12 @@ handle_audio_fields (ConvertData *d)
       if (i == 1)
         choice->body.type = SPA_CHOICE_None;
     }
+  } else if (strcmp(d->type->name, "audio/x-mulaw") == 0) {
+        spa_pod_builder_prop (&d->b, SPA_FORMAT_AUDIO_format, 0);
+        spa_pod_builder_id (&d->b, SPA_AUDIO_FORMAT_ULAW);
+  } else if (strcmp(d->type->name, "audio/x-alaw") == 0) {
+        spa_pod_builder_prop (&d->b, SPA_FORMAT_AUDIO_format, 0);
+        spa_pod_builder_id (&d->b, SPA_AUDIO_FORMAT_ALAW);
   }
 
 #if 0
@@ -478,9 +541,9 @@ handle_audio_fields (ConvertData *d)
     for (i = 0; (v = get_nth_string (value, i)); i++) {
       enum spa_audio_layout layout;
 
-      if (!strcmp (v, "interleaved"))
+      if (spa_streq(v, "interleaved"))
         layout = SPA_AUDIO_LAYOUT_INTERLEAVED;
-      else if (!strcmp (v, "non-interleaved"))
+      else if (spa_streq(v, "non-interleaved"))
         layout = SPA_AUDIO_LAYOUT_NON_INTERLEAVED;
       else
         break;
@@ -529,8 +592,10 @@ handle_audio_fields (ConvertData *d)
     }
     if (i > 0) {
       choice = spa_pod_builder_pop(&d->b, &f);
-      if (i == 1)
+      if (i == 1) {
         choice->body.type = SPA_CHOICE_None;
+        set_default_channels (&d->b, v);
+      }
     }
   }
   return TRUE;
@@ -570,6 +635,18 @@ convert_1 (ConvertData *d)
   spa_pod_builder_prop (&d->b, SPA_FORMAT_mediaSubtype, 0);
   spa_pod_builder_id(&d->b, d->type->media_subtype);
 
+  if (d->cf && gst_caps_features_contains (d->cf, GST_CAPS_FEATURE_MEMORY_DMABUF)) {
+    struct spa_pod_frame f2;
+
+    spa_pod_builder_prop (&d->b, SPA_FORMAT_VIDEO_modifier,
+                          (SPA_POD_PROP_FLAG_MANDATORY | SPA_POD_PROP_FLAG_DONT_FIXATE));
+    spa_pod_builder_push_choice (&d->b, &f2, SPA_CHOICE_Enum, 0);
+    spa_pod_builder_long (&d->b, DRM_FORMAT_MOD_INVALID);
+    spa_pod_builder_long (&d->b, DRM_FORMAT_MOD_INVALID);
+    spa_pod_builder_long (&d->b, DRM_FORMAT_MOD_LINEAR);
+    spa_pod_builder_pop (&d->b, &f2);
+  }
+
   if (d->type->media_type == SPA_MEDIA_TYPE_video)
     handle_video_fields (d);
   else if (d->type->media_type == SPA_MEDIA_TYPE_audio)
@@ -577,7 +654,7 @@ convert_1 (ConvertData *d)
 
   spa_pod_builder_pop (&d->b, &f);
 
-  return SPA_MEMBER (d->b.data, 0, struct spa_pod);
+  return SPA_PTROFF (d->b.data, 0, struct spa_pod);
 }
 
 struct spa_pod *
@@ -605,13 +682,19 @@ foreach_func (GstCapsFeatures *features,
               ConvertData     *d)
 {
   struct spa_pod *fmt;
+  int idx;
 
   spa_zero(d->b);
   d->cf = features;
   d->cs = structure;
 
+  if (d->cf && gst_caps_features_contains (d->cf, GST_CAPS_FEATURE_MEMORY_DMABUF))
+    idx = 0;
+  else
+    idx = -1;
+
   if ((fmt = convert_1 (d)))
-    g_ptr_array_insert (d->array, -1, fmt);
+    g_ptr_array_insert (d->array, idx, fmt);
 
   return TRUE;
 }
@@ -885,8 +968,6 @@ gst_caps_from_format (const struct spa_pod *format)
         handle_int_prop (prop, "channels", res);
       }
     }
-    else if (media_subtype == SPA_MEDIA_SUBTYPE_aac) {
-    }
   }
   return res;
 }
