@@ -33,6 +33,8 @@
  * </refsect2>
  */
 
+#define PW_ENABLE_DEPRECATED
+
 #include "config.h"
 #include "gstpipewiresink.h"
 
@@ -582,6 +584,8 @@ gst_pipewire_sink_setcaps (GstBaseSink * bsink, GstCaps * caps)
     if (pwsink->target_object) {
       struct spa_dict_item items[2] = {
         SPA_DICT_ITEM_INIT(PW_KEY_TARGET_OBJECT, pwsink->target_object),
+	/* XXX deprecated but the portal and some example apps only
+	 * provide the object id */
         SPA_DICT_ITEM_INIT(PW_KEY_NODE_TARGET, NULL),
       };
       struct spa_dict dict = SPA_DICT_INIT_ARRAY(items);
