@@ -33,6 +33,8 @@
  * </refsect2>
  */
 
+#define PW_ENABLE_DEPRECATED
+
 #include "config.h"
 #include "gstpipewiresrc.h"
 #include "gstpipewireformat.h"
@@ -43,7 +45,9 @@
 #include <sys/socket.h>
 #include <unistd.h>
 
+#include <spa/param/video/format.h>
 #include <spa/pod/builder.h>
+#include <spa/utils/result.h>
 
 #include <gst/net/gstnetclientclock.h>
 #include <gst/allocators/gstfdmemory.h>
@@ -443,6 +447,7 @@ buffer_recycle (GstMiniObject *obj)
 {
   GstPipeWireSrc *src;
   GstPipeWirePoolData *data;
+  int res;
 
   data = gst_pipewire_pool_get_data (GST_BUFFER_CAST(obj));
 
@@ -466,8 +471,11 @@ buffer_recycle (GstMiniObject *obj)
 
   data->queued = TRUE;
 
-  GST_LOG_OBJECT (src, "recycle buffer %p", obj);
-  pw_stream_queue_buffer (src->stream, data->b);
+  if ((res = pw_stream_queue_buffer (src->stream, data->b)) < 0)
+    GST_WARNING_OBJECT (src, "can't queue recycled buffer %p, %s", obj, spa_strerror(res));
+  else
+    GST_LOG_OBJECT (src, "recycle buffer %p", obj);
+
   pw_thread_loop_unlock (src->core->loop);
 
   GST_OBJECT_UNLOCK (data->pool);
@@ -481,9 +489,9 @@ on_add_buffer (void *_data, struct pw_buffer *b)
   GstPipeWireSrc *pwsrc = _data;
   GstPipeWirePoolData *data;
 
-  GST_DEBUG_OBJECT (pwsrc, "add buffer");
   gst_pipewire_pool_wrap_buffer (pwsrc->pool, b);
   data = b->user_data;
+  GST_DEBUG_OBJECT (pwsrc, "add buffer %p", data->buf);
   data->owner = pwsrc;
   data->queued = TRUE;
   GST_MINI_OBJECT_CAST (data->buf)->dispose = buffer_recycle;
@@ -495,15 +503,37 @@ on_remove_buffer (void *_data, struct pw_buffer *b)
   GstPipeWireSrc *pwsrc = _data;
   GstPipeWirePoolData *data = b->user_data;
   GstBuffer *buf = data->buf;
+  int res;
 
   GST_DEBUG_OBJECT (pwsrc, "remove buffer %p", buf);
 
   GST_MINI_OBJECT_CAST (buf)->dispose = NULL;
 
-  if (data->queued)
+  if (data->queued) {
     gst_buffer_unref (buf);
-  else
-    pw_stream_queue_buffer (pwsrc->stream, b);
+  } else {
+    if ((res = pw_stream_queue_buffer (pwsrc->stream, b)) < 0)
+      GST_WARNING_OBJECT (pwsrc, "can't queue removed buffer %p, %s", buf, spa_strerror(res));
+  }
+}
+
+static const char * const transform_map[] = {
+  [SPA_META_TRANSFORMATION_None] = "rotate-0",
+  [SPA_META_TRANSFORMATION_90] = "rotate-90",
+  [SPA_META_TRANSFORMATION_180] = "rotate-180",
+  [SPA_META_TRANSFORMATION_270] = "rotate-270",
+  [SPA_META_TRANSFORMATION_Flipped] = "flip-rotate-0",
+  [SPA_META_TRANSFORMATION_Flipped90] = "flip-rotate-270",
+  [SPA_META_TRANSFORMATION_Flipped180] = "flip-rotate-180",
+  [SPA_META_TRANSFORMATION_Flipped270] = "flip-rotate-90",
+};
+
+static const char *spa_transform_value_to_gst_image_orientation(uint32_t transform_value)
+{
+  if (transform_value >= SPA_N_ELEMENTS(transform_map))
+    transform_value = SPA_META_TRANSFORMATION_None;
+
+  return transform_map[transform_value];
 }
 
 static GstBuffer *dequeue_buffer(GstPipeWireSrc *pwsrc)
@@ -513,6 +543,7 @@ static GstBuffer *dequeue_buffer(GstPipeWireSrc *pwsrc)
   GstPipeWirePoolData *data;
   struct spa_meta_header *h;
   struct spa_meta_region *crop;
+  struct spa_meta_videotransform *videotransform;
   guint i;
 
   b = pw_stream_dequeue_buffer (pwsrc->stream);
@@ -560,19 +591,44 @@ static GstBuffer *dequeue_buffer(GstPipeWireSrc *pwsrc)
       meta->height = crop->region.size.height;
     }
   }
-  gst_buffer_add_parent_buffer_meta (buf, data->buf);
-  gst_buffer_unref (data->buf);
+
+  videotransform = data->videotransform;
+  if (videotransform) {
+    if (pwsrc->transform_value != videotransform->transform) {
+      GstEvent *tag_event;
+      const char* tag_string;
+
+      tag_string =
+          spa_transform_value_to_gst_image_orientation(videotransform->transform);
+
+      GST_LOG_OBJECT (pwsrc, "got new videotransform: %u / %s",
+          videotransform->transform, tag_string);
+
+      tag_event = gst_event_new_tag(gst_tag_list_new(GST_TAG_IMAGE_ORIENTATION,
+          tag_string, NULL));
+      gst_pad_push_event (GST_BASE_SRC_PAD (pwsrc), tag_event);
+
+      pwsrc->transform_value = videotransform->transform;
+    }
+  }
+
   for (i = 0; i < b->buffer->n_datas; i++) {
     struct spa_data *d = &b->buffer->datas[i];
     GstMemory *pmem = gst_buffer_peek_memory (data->buf, i);
     if (pmem) {
-      GstMemory *mem = gst_memory_share (pmem, d->chunk->offset, d->chunk->size);
+      GstMemory *mem;
+      if (!pwsrc->always_copy)
+        mem = gst_memory_share (pmem, d->chunk->offset, d->chunk->size);
+      else
+        mem = gst_memory_copy (pmem, d->chunk->offset, d->chunk->size);
       gst_buffer_insert_memory (buf, i, mem);
-      spa_assert_se(mem->size <= mem->maxsize);
     }
     if (d->chunk->flags & SPA_CHUNK_FLAG_CORRUPTED)
       GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_CORRUPTED);
   }
+  if (!pwsrc->always_copy)
+    gst_buffer_add_parent_buffer_meta (buf, data->buf);
+  gst_buffer_unref (data->buf);
   return buf;
 }
 
@@ -788,6 +844,8 @@ gst_pipewire_src_negotiate (GstBaseSrc * basesrc)
   if (pwsrc->target_object) {
       struct spa_dict_item items[2] = {
         SPA_DICT_ITEM_INIT(PW_KEY_TARGET_OBJECT, pwsrc->target_object),
+	/* XXX deprecated but the portal and some example apps only
+	 * provide the object id */
         SPA_DICT_ITEM_INIT(PW_KEY_NODE_TARGET, NULL),
       };
       struct spa_dict dict = SPA_DICT_INIT_ARRAY(items);
@@ -896,15 +954,27 @@ on_param_changed (void *data, uint32_t id,
   if (pwsrc->caps)
           gst_caps_unref(pwsrc->caps);
   pwsrc->caps = gst_caps_from_format (param);
-  GST_DEBUG_OBJECT (pwsrc, "we got format %" GST_PTR_FORMAT, pwsrc->caps);
 
   pwsrc->negotiated = pwsrc->caps != NULL;
 
   if (pwsrc->negotiated) {
-    const struct spa_pod *params[3];
+    const struct spa_pod *params[4];
     struct spa_pod_builder b = { NULL };
     uint8_t buffer[512];
     uint32_t buffers = CLAMP (16, pwsrc->min_buffers, pwsrc->max_buffers);
+    int buffertypes;
+
+    if (spa_pod_find_prop (param, NULL, SPA_FORMAT_VIDEO_modifier)) {
+      buffertypes = (1<<SPA_DATA_DmaBuf);
+      gst_caps_features_remove (gst_caps_get_features (pwsrc->caps, 0),
+          GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY);
+      gst_caps_features_add (gst_caps_get_features (pwsrc->caps, 0),
+          GST_CAPS_FEATURE_MEMORY_DMABUF);
+    } else {
+      buffertypes = ((1<<SPA_DATA_MemFd) | (1<<SPA_DATA_MemPtr));
+    }
+
+    GST_DEBUG_OBJECT (pwsrc, "we got format %" GST_PTR_FORMAT, pwsrc->caps);
 
     spa_pod_builder_init (&b, buffer, sizeof (buffer));
     params[0] = spa_pod_builder_add_object (&b,
@@ -915,9 +985,7 @@ on_param_changed (void *data, uint32_t id,
         SPA_PARAM_BUFFERS_blocks,  SPA_POD_CHOICE_RANGE_Int(0, 1, INT32_MAX),
         SPA_PARAM_BUFFERS_size,    SPA_POD_CHOICE_RANGE_Int(0, 0, INT32_MAX),
         SPA_PARAM_BUFFERS_stride,  SPA_POD_CHOICE_RANGE_Int(0, 0, INT32_MAX),
-        SPA_PARAM_BUFFERS_dataType, SPA_POD_CHOICE_FLAGS_Int(
-                                                (1<<SPA_DATA_MemFd) |
-                                                (1<<SPA_DATA_MemPtr)));
+        SPA_PARAM_BUFFERS_dataType, SPA_POD_CHOICE_FLAGS_Int(buffertypes));
 
     params[1] = spa_pod_builder_add_object (&b,
         SPA_TYPE_OBJECT_ParamMeta, SPA_PARAM_Meta,
@@ -927,9 +995,13 @@ on_param_changed (void *data, uint32_t id,
         SPA_TYPE_OBJECT_ParamMeta, SPA_PARAM_Meta,
         SPA_PARAM_META_type, SPA_POD_Id(SPA_META_VideoCrop),
         SPA_PARAM_META_size, SPA_POD_Int(sizeof (struct spa_meta_region)));
+    params[3] = spa_pod_builder_add_object (&b,
+        SPA_TYPE_OBJECT_ParamMeta, SPA_PARAM_Meta,
+        SPA_PARAM_META_type, SPA_POD_Id(SPA_META_VideoTransform),
+        SPA_PARAM_META_size, SPA_POD_Int(sizeof (struct spa_meta_videotransform)));
 
     GST_DEBUG_OBJECT (pwsrc, "doing finish format");
-    pw_stream_update_params (pwsrc->stream, params, 3);
+    pw_stream_update_params (pwsrc->stream, params, SPA_N_ELEMENTS(params));
   } else {
     GST_WARNING_OBJECT (pwsrc, "finish format with error");
     pw_stream_set_error (pwsrc->stream, -EINVAL, "unhandled format");
@@ -1026,10 +1098,10 @@ gst_pipewire_src_create (GstPushSrc * psrc, GstBuffer ** buffer)
 
   pwsrc = GST_PIPEWIRE_SRC (psrc);
 
+  pw_thread_loop_lock (pwsrc->core->loop);
   if (!pwsrc->negotiated)
     goto not_negotiated;
 
-  pw_thread_loop_lock (pwsrc->core->loop);
   while (TRUE) {
     enum pw_stream_state state;
 
@@ -1083,12 +1155,7 @@ gst_pipewire_src_create (GstPushSrc * psrc, GstBuffer ** buffer)
   }
   pw_thread_loop_unlock (pwsrc->core->loop);
 
-  if (pwsrc->always_copy) {
-    *buffer = gst_buffer_copy_deep (buf);
-    gst_buffer_unref (buf);
-  }
-  else
-    *buffer = buf;
+  *buffer = buf;
 
   if (pwsrc->is_live)
     base_time = GST_ELEMENT_CAST (psrc)->base_time;
@@ -1126,6 +1193,7 @@ gst_pipewire_src_create (GstPushSrc * psrc, GstBuffer ** buffer)
 
 not_negotiated:
   {
+    pw_thread_loop_unlock (pwsrc->core->loop);
     return GST_FLOW_NOT_NEGOTIATED;
   }
 streaming_eos:
@@ -1348,7 +1416,9 @@ gst_pipewire_src_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+      pw_thread_loop_lock (this->core->loop);
       this->negotiated = FALSE;
+      pw_thread_loop_unlock (this->core->loop);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       gst_pipewire_src_close (this);
