@@ -43,6 +43,7 @@
 #include <spa/param/audio/raw.h>
 #include <spa/param/profiler.h>
 #include <spa/pod/builder.h>
+#include <spa/pod/dynamic.h>
 #include <spa/support/plugin.h>
 #include <spa/utils/json.h>
 #include <spa/utils/names.h>
@@ -68,8 +69,10 @@
  *
  * Options specific to the behavior of this module
  *
+ * - `capture.props = {}`: properties to be passed to the capture stream
  * - `source.props = {}`: properties to be passed to the source stream
  * - `sink.props = {}`: properties to be passed to the sink stream
+ * - `playback.props = {}`: properties to be passed to the playback stream
  * - `library.name = <str>`: the echo cancellation library  Currently supported:
  * `aec/libspa-aec-webrtc`. Leave unset to use the default method (`aec/libspa-aec-webrtc`).
  * - `aec.args = <str>`: arguments to pass to the echo cancellation method
@@ -80,6 +83,7 @@
  *
  * - \ref PW_KEY_AUDIO_RATE
  * - \ref PW_KEY_AUDIO_CHANNELS
+ * - \ref SPA_KEY_AUDIO_POSITION
  * - \ref PW_KEY_MEDIA_CLASS
  * - \ref PW_KEY_NODE_LATENCY
  * - \ref PW_KEY_NODE_NAME
@@ -89,7 +93,6 @@
  * - \ref PW_KEY_NODE_VIRTUAL
  * - \ref PW_KEY_NODE_LATENCY
  * - \ref PW_KEY_REMOTE_NAME
- * - \ref SPA_KEY_AUDIO_POSITION
  *
  * ## Example configuration
  *\code{.unparsed}
@@ -98,12 +101,18 @@
  *      args = {
  *          # library.name  = aec/libspa-aec-webrtc
  *          # node.latency = 1024/48000
+ *          capture.props = {
+ *             node.name = "Echo Cancellation Capture"
+ *          }
  *          source.props = {
  *             node.name = "Echo Cancellation Source"
  *          }
  *          sink.props = {
  *             node.name = "Echo Cancellation Sink"
  *          }
+ *          playback.props = {
+ *             node.name = "Echo Cancellation Playback"
+ *          }
  *      }
  *  }
  *]
@@ -125,6 +134,10 @@
 PW_LOG_TOPIC_STATIC(mod_topic, "mod." NAME);
 #define PW_LOG_TOPIC_DEFAULT mod_topic
 
+#define DEFAULT_RATE 48000
+#define DEFAULT_CHANNELS 2
+#define DEFAULT_POSITION "[ FL FR ]"
+
 /* Hopefully this is enough for any combination of AEC engine and resampler
  * input requirement for rate matching */
 #define MAX_BUFSIZE_MS 100
@@ -142,8 +155,10 @@ static const struct spa_dict_item module_props[] = {
 				"[ buffer.play_delay=<delay as fraction> ] "
 				"[ library.name =<library name> ] "
 				"[ aec.args=<aec arguments> ] "
+				"[ capture.props=<properties> ] "
 				"[ source.props=<properties> ] "
-				"[ sink.props=<properties> ] " },
+				"[ sink.props=<properties> ] "
+				"[ playback.props=<properties> ] " },
 	{ PW_KEY_MODULE_VERSION, PACKAGE_VERSION },
 };
 
@@ -159,17 +174,22 @@ struct impl {
 
 	struct spa_audio_info_raw info;
 
+	struct pw_properties *capture_props;
 	struct pw_stream *capture;
 	struct spa_hook capture_listener;
+
 	struct pw_properties *source_props;
 	struct pw_stream *source;
 	struct spa_hook source_listener;
+
 	void *rec_buffer[SPA_AUDIO_MAX_CHANNELS];
 	uint32_t rec_ringsize;
 	struct spa_ringbuffer rec_ring;
 
+	struct pw_properties *playback_props;
 	struct pw_stream *playback;
 	struct spa_hook playback_listener;
+
 	struct pw_properties *sink_props;
 	struct pw_stream *sink;
 	struct spa_hook sink_listener;
@@ -192,15 +212,18 @@ struct impl {
 
 	uint32_t max_buffer_size;
 	uint32_t buffer_delay;
+	uint32_t current_delay;
 
 	struct spa_handle *spa_handle;
 	struct spa_plugin_loader *loader;
+
+	bool monitor_mode;
 };
 
 static void process(struct impl *impl)
 {
 	struct pw_buffer *cout;
-	struct pw_buffer *pout;
+	struct pw_buffer *pout = NULL;
 	float rec_buf[impl->info.channels][impl->aec_blocksize / sizeof(float)];
 	float play_buf[impl->info.channels][impl->aec_blocksize / sizeof(float)];
 	float play_delayed_buf[impl->info.channels][impl->aec_blocksize / sizeof(float)];
@@ -214,7 +237,7 @@ static void process(struct impl *impl)
 	uint32_t rindex, pindex, oindex, pdindex, avail;
 	int32_t stride = 0;
 
-	if ((pout = pw_stream_dequeue_buffer(impl->playback)) == NULL) {
+	if (impl->playback != NULL && (pout = pw_stream_dequeue_buffer(impl->playback)) == NULL) {
 		pw_log_debug("out of playback buffers: %m");
 		goto done;
 	}
@@ -252,23 +275,50 @@ static void process(struct impl *impl)
 				impl->play_ringsize, pdindex % impl->play_ringsize,
 				(void *)play_delayed[i], size);
 
-		/* output to sink, just copy */
-		dd = &pout->buffer->datas[i];
-		memcpy(dd->data, play[i], size);
+		if (pout != NULL) {
+			/* output to sink, just copy */
+			dd = &pout->buffer->datas[i];
+			memcpy(dd->data, play[i], size);
 
-		dd->chunk->offset = 0;
-		dd->chunk->size = size;
-		dd->chunk->stride = stride;
+			dd->chunk->offset = 0;
+			dd->chunk->size = size;
+			dd->chunk->stride = stride;
+		}
 	}
 
 	spa_ringbuffer_read_update(&impl->rec_ring, rindex + size);
 	spa_ringbuffer_read_update(&impl->play_ring, pindex + size);
 	spa_ringbuffer_read_update(&impl->play_delayed_ring, pdindex + size);
 
-	pw_stream_queue_buffer(impl->playback, pout);
+	if (impl->playback != NULL)
+		pw_stream_queue_buffer(impl->playback, pout);
 
-	/* Now run the canceller */
-	spa_audio_aec_run(impl->aec, rec, play_delayed, out, size / sizeof(float));
+	if (SPA_UNLIKELY (impl->current_delay < impl->buffer_delay)) {
+		uint32_t delay_left = impl->buffer_delay - impl->current_delay;
+		uint32_t silence_size;
+
+		/* don't run the canceller until play_buffer has been filled,
+		 * copy silence to output in the meantime */
+		silence_size = SPA_MIN(size, delay_left * sizeof(float));
+		for (i = 0; i < impl->info.channels; i++)
+			memset(out[i], 0, silence_size);
+		impl->current_delay += silence_size / sizeof(float);
+		pw_log_debug("current_delay %d", impl->current_delay);
+
+		if (silence_size != size) {
+			const float *pd[impl->info.channels];
+			float *o[impl->info.channels];
+
+			for (i = 0; i < impl->info.channels; i++) {
+				pd[i] = play_delayed[i] + delay_left;
+				o[i] = out[i] + delay_left;
+			}
+			spa_audio_aec_run(impl->aec, rec, pd, o, size / sizeof(float) - delay_left);
+		}
+	} else {
+		/* run the canceller */
+		spa_audio_aec_run(impl->aec, rec, play_delayed, out, size / sizeof(float));
+	}
 
 	/* Next, copy over the output to the output ringbuffer */
 	avail = spa_ringbuffer_get_write_index(&impl->out_ring, &oindex);
@@ -398,7 +448,7 @@ static void capture_process(void *data)
 	pw_stream_queue_buffer(impl->capture, buf);
 }
 
-static void input_state_changed(void *data, enum pw_stream_state old,
+static void capture_state_changed(void *data, enum pw_stream_state old,
 		enum pw_stream_state state, const char *error)
 {
 	struct impl *impl = data;
@@ -419,6 +469,44 @@ static void input_state_changed(void *data, enum pw_stream_state old,
 	}
 }
 
+static void source_state_changed(void *data, enum pw_stream_state old,
+		enum pw_stream_state state, const char *error)
+{
+	struct impl *impl = data;
+	int res;
+
+	switch (state) {
+	case PW_STREAM_STATE_PAUSED:
+		pw_stream_flush(impl->source, false);
+		pw_stream_flush(impl->capture, false);
+
+		if (old == PW_STREAM_STATE_STREAMING) {
+			pw_log_debug("%p: deactivate %s", impl, impl->aec->name);
+			res = spa_audio_aec_deactivate(impl->aec);
+			if (res < 0 && res != -EOPNOTSUPP) {
+				pw_log_error("aec plugin %s deactivate failed: %s", impl->aec->name, spa_strerror(res));
+			}
+		}
+		break;
+	case PW_STREAM_STATE_STREAMING:
+		pw_log_debug("%p: activate %s", impl, impl->aec->name);
+		res = spa_audio_aec_activate(impl->aec);
+		if (res < 0 && res != -EOPNOTSUPP) {
+			pw_log_error("aec plugin %s activate failed: %s", impl->aec->name, spa_strerror(res));
+		}
+		break;
+	case PW_STREAM_STATE_UNCONNECTED:
+		pw_log_info("%p: input unconnected", impl);
+		pw_impl_module_schedule_destroy(impl->module);
+		break;
+	case PW_STREAM_STATE_ERROR:
+		pw_log_info("%p: input error: %s", impl, error);
+		break;
+	default:
+		break;
+	}
+}
+
 static void input_param_latency_changed(struct impl *impl, const struct spa_pod *param)
 {
 	struct spa_latency_info latency;
@@ -437,21 +525,62 @@ static void input_param_latency_changed(struct impl *impl, const struct spa_pod
 	else
 		pw_stream_update_params(impl->capture, params, 1);
 }
-
-static void input_param_changed(void *data, uint32_t id, const struct spa_pod *param)
+static struct spa_pod* get_props_param(struct impl* impl, struct spa_pod_builder* b)
 {
-	struct impl *impl = data;
+	if (spa_audio_aec_get_params(impl->aec, NULL) > 0) {
+		struct spa_pod_frame f[2];
+		spa_pod_builder_push_object(
+		    b, &f[0], SPA_TYPE_OBJECT_Props, SPA_PARAM_Props);
+		spa_pod_builder_prop(b, SPA_PROP_params, 0);
+		spa_pod_builder_push_struct(b, &f[1]);
+
+		spa_audio_aec_get_params(impl->aec, b);
+
+		spa_pod_builder_pop(b, &f[1]);
+		return spa_pod_builder_pop(b, &f[0]);
+	}
+	return NULL;
+}
+
+static void input_param_changed(void *data, uint32_t id, const struct spa_pod* param)
+{
+	struct spa_pod_object* obj = (struct spa_pod_object*)param;
+	const struct spa_pod_prop* prop;
+	struct impl* impl = data;
 	switch (id) {
 	case SPA_PARAM_Latency:
 		input_param_latency_changed(impl, param);
 		break;
+	case SPA_PARAM_Props:
+		if (param != NULL) {
+			uint8_t buffer[1024];
+			struct spa_pod_dynamic_builder b;
+			const struct spa_pod* params[1];
+			SPA_POD_OBJECT_FOREACH(obj, prop)
+			{
+				if (prop->key == SPA_PROP_params) {
+					spa_audio_aec_set_params(impl->aec, &prop->value);
+				}
+			}
+
+			spa_pod_dynamic_builder_init(&b, buffer, sizeof(buffer), 4096);
+			params[0] = get_props_param(impl, &b.b);
+			if (params[0]) {
+				pw_stream_update_params(impl->capture, params, 1);
+				pw_stream_update_params(impl->playback, params, 1);
+			}
+			spa_pod_dynamic_builder_clean(&b);
+		} else {
+			pw_log_warn("param is null");
+		}
+		break;
 	}
 }
 
 static const struct pw_stream_events capture_events = {
 	PW_VERSION_STREAM_EVENTS,
 	.destroy = capture_destroy,
-	.state_changed = input_state_changed,
+	.state_changed = capture_state_changed,
 	.process = capture_process,
 	.param_changed = input_param_changed
 };
@@ -466,7 +595,7 @@ static void source_destroy(void *d)
 static const struct pw_stream_events source_events = {
 	PW_VERSION_STREAM_EVENTS,
 	.destroy = source_destroy,
-	.state_changed = input_state_changed,
+	.state_changed = source_state_changed,
 	.param_changed = input_param_changed
 };
 
@@ -477,7 +606,11 @@ static void output_state_changed(void *data, enum pw_stream_state old,
 	switch (state) {
 	case PW_STREAM_STATE_PAUSED:
 		pw_stream_flush(impl->sink, false);
-		pw_stream_flush(impl->playback, false);
+		if (impl->playback != NULL)
+			pw_stream_flush(impl->playback, false);
+		if (old == PW_STREAM_STATE_STREAMING) {
+			impl->current_delay = 0;
+		}
 		break;
 	case PW_STREAM_STATE_UNCONNECTED:
 		pw_log_info("%p: output unconnected", impl);
@@ -506,17 +639,40 @@ static void output_param_latency_changed(struct impl *impl, const struct spa_pod
 
 	if (latency.direction == SPA_DIRECTION_INPUT)
 		pw_stream_update_params(impl->sink, params, 1);
-	else
+	else if (impl->playback != NULL)
 		pw_stream_update_params(impl->playback, params, 1);
 }
 
 static void output_param_changed(void *data, uint32_t id, const struct spa_pod *param)
 {
+	struct spa_pod_object *obj = (struct spa_pod_object *) param;
+	const struct spa_pod_prop *prop;
 	struct impl *impl = data;
 	switch (id) {
 	case SPA_PARAM_Latency:
 		output_param_latency_changed(impl, param);
 		break;
+	case SPA_PARAM_Props:
+		if (param != NULL) {
+			uint8_t buffer[1024];
+			struct spa_pod_dynamic_builder b;
+			const struct spa_pod* params[1];
+
+			SPA_POD_OBJECT_FOREACH(obj, prop)
+			{
+				if (prop->key == SPA_PROP_params) {
+					spa_audio_aec_set_params(impl->aec, &prop->value);
+				}
+			}
+			spa_pod_dynamic_builder_init(&b, buffer, sizeof(buffer), 4096);
+			params[0] = get_props_param(impl, &b.b);
+			if (params[0] != NULL) {
+				pw_stream_update_params(impl->capture, params, 1);
+				pw_stream_update_params(impl->playback, params, 1);
+			}
+			spa_pod_dynamic_builder_clean(&b);
+		}
+		break;
 	}
 }
 
@@ -593,8 +749,10 @@ static void sink_process(void *data)
 static void playback_destroy(void *d)
 {
 	struct impl *impl = d;
-	spa_hook_remove(&impl->playback_listener);
-	impl->playback = NULL;
+	if (impl->playback != NULL) {
+		spa_hook_remove(&impl->playback_listener);
+		impl->playback = NULL;
+	}
 }
 
 static const struct pw_stream_events playback_events = {
@@ -615,33 +773,14 @@ static int setup_streams(struct impl *impl)
 {
 	int res;
 	uint32_t n_params, i;
-	const struct spa_pod *params[1];
-	uint8_t buffer[1024];
-	struct spa_pod_builder b;
-	struct pw_properties *props;
-	const char *str;
+	uint32_t offsets[512];
+	const struct spa_pod *params[512];
+	struct spa_pod_dynamic_builder b;
 	uint32_t index;
 
-	props = pw_properties_new(
-			PW_KEY_NODE_NAME, "echo-cancel-capture",
-			PW_KEY_NODE_VIRTUAL, "true",
-			PW_KEY_NODE_PASSIVE, "true",
-			NULL);
-	if ((str = pw_properties_get(impl->source_props, PW_KEY_NODE_GROUP)) != NULL)
-		pw_properties_set(props, PW_KEY_NODE_GROUP, str);
-	if ((str = pw_properties_get(impl->source_props, PW_KEY_NODE_LINK_GROUP)) != NULL)
-		pw_properties_set(props, PW_KEY_NODE_LINK_GROUP, str);
-	if ((str = pw_properties_get(impl->source_props, PW_KEY_NODE_LATENCY)) != NULL)
-		pw_properties_set(props, PW_KEY_NODE_LATENCY, str);
-	else if (impl->aec->latency)
-		pw_properties_set(props, PW_KEY_NODE_LATENCY, impl->aec->latency);
-	if ((str = pw_properties_get(impl->source_props, SPA_KEY_AUDIO_CHANNELS)) != NULL)
-		pw_properties_set(props, SPA_KEY_AUDIO_CHANNELS, str);
-	if ((str = pw_properties_get(impl->source_props, SPA_KEY_AUDIO_POSITION)) != NULL)
-		pw_properties_set(props, SPA_KEY_AUDIO_POSITION, str);
-
 	impl->capture = pw_stream_new(impl->core,
-			"Echo-Cancel Capture", props);
+			"Echo-Cancel Capture", impl->capture_props);
+	impl->capture_props = NULL;
 	if (impl->capture == NULL)
 		return -errno;
 
@@ -659,32 +798,19 @@ static int setup_streams(struct impl *impl)
 			&impl->source_listener,
 			&source_events, impl);
 
-	props = pw_properties_new(
-			PW_KEY_NODE_NAME, "echo-cancel-playback",
-			PW_KEY_NODE_VIRTUAL, "true",
-			PW_KEY_NODE_PASSIVE, "true",
-			NULL);
-	if ((str = pw_properties_get(impl->sink_props, PW_KEY_NODE_GROUP)) != NULL)
-		pw_properties_set(props, PW_KEY_NODE_GROUP, str);
-	if ((str = pw_properties_get(impl->sink_props, PW_KEY_NODE_LINK_GROUP)) != NULL)
-		pw_properties_set(props, PW_KEY_NODE_LINK_GROUP, str);
-	if ((str = pw_properties_get(impl->sink_props, PW_KEY_NODE_LATENCY)) != NULL)
-		pw_properties_set(props, PW_KEY_NODE_LATENCY, str);
-	else if (impl->aec->latency)
-		pw_properties_set(props, PW_KEY_NODE_LATENCY, impl->aec->latency);
-	if ((str = pw_properties_get(impl->sink_props, SPA_KEY_AUDIO_CHANNELS)) != NULL)
-		pw_properties_set(props, SPA_KEY_AUDIO_CHANNELS, str);
-	if ((str = pw_properties_get(impl->sink_props, SPA_KEY_AUDIO_POSITION)) != NULL)
-		pw_properties_set(props, SPA_KEY_AUDIO_POSITION, str);
+	if (impl->monitor_mode) {
+		impl->playback = NULL;
+	} else {
+		impl->playback = pw_stream_new(impl->core,
+				"Echo-Cancel Playback", impl->playback_props);
+		impl->playback_props = NULL;
+		if (impl->playback == NULL)
+			return -errno;
 
-	impl->playback = pw_stream_new(impl->core,
-			"Echo-Cancel Playback", props);
-	if (impl->playback == NULL)
-		return -errno;
-
-	pw_stream_add_listener(impl->playback,
-			&impl->playback_listener,
-			&playback_events, impl);
+		pw_stream_add_listener(impl->playback,
+				&impl->playback_listener,
+				&playback_events, impl);
+	}
 
 	impl->sink = pw_stream_new(impl->core,
 			"Echo-Cancel Sink", impl->sink_props);
@@ -697,9 +823,23 @@ static int setup_streams(struct impl *impl)
 			&sink_events, impl);
 
 	n_params = 0;
-	spa_pod_builder_init(&b, buffer, sizeof(buffer));
-	params[n_params++] = spa_format_audio_raw_build(&b, SPA_PARAM_EnumFormat,
-			&impl->info);
+	spa_pod_dynamic_builder_init(&b, NULL, 0, 4096);
+
+	offsets[n_params++] = b.b.state.offset;
+	spa_format_audio_raw_build(&b.b, SPA_PARAM_EnumFormat, &impl->info);
+
+	int nbr_of_external_props = spa_audio_aec_enum_props(impl->aec, 0, NULL);
+	if (nbr_of_external_props > 0) {
+		for (int i = 0; i < nbr_of_external_props; i++) {
+			offsets[n_params++] = b.b.state.offset;
+			spa_audio_aec_enum_props(impl->aec, i, &b.b);
+		}
+		get_props_param(impl, &b.b);
+	}
+
+	for (i = 0; i < n_params; i++) {
+		params[i] = spa_pod_builder_deref(&b.b, offsets[i]);
+	}
 
 	if ((res = pw_stream_connect(impl->capture,
 			PW_DIRECTION_INPUT,
@@ -707,33 +847,44 @@ static int setup_streams(struct impl *impl)
 			PW_STREAM_FLAG_AUTOCONNECT |
 			PW_STREAM_FLAG_MAP_BUFFERS |
 			PW_STREAM_FLAG_RT_PROCESS,
-			params, n_params)) < 0)
+			params, n_params)) < 0) {
+		spa_pod_dynamic_builder_clean(&b);
 		return res;
+	}
 
 	if ((res = pw_stream_connect(impl->source,
 			PW_DIRECTION_OUTPUT,
 			PW_ID_ANY,
 			PW_STREAM_FLAG_MAP_BUFFERS |
 			PW_STREAM_FLAG_RT_PROCESS,
-			params, n_params)) < 0)
+			params, n_params)) < 0) {
+		spa_pod_dynamic_builder_clean(&b);
 		return res;
+	}
 
 	if ((res = pw_stream_connect(impl->sink,
 			PW_DIRECTION_INPUT,
 			PW_ID_ANY,
-			PW_STREAM_FLAG_MAP_BUFFERS |
-			PW_STREAM_FLAG_RT_PROCESS,
-			params, n_params)) < 0)
+			impl->playback != NULL ?
+				PW_STREAM_FLAG_MAP_BUFFERS | PW_STREAM_FLAG_RT_PROCESS :
+				PW_STREAM_FLAG_AUTOCONNECT | PW_STREAM_FLAG_MAP_BUFFERS | PW_STREAM_FLAG_RT_PROCESS,
+			params, n_params)) < 0) {
+		spa_pod_dynamic_builder_clean(&b);
 		return res;
+	}
 
-	if ((res = pw_stream_connect(impl->playback,
+	if (impl->playback != NULL && (res = pw_stream_connect(impl->playback,
 			PW_DIRECTION_OUTPUT,
 			PW_ID_ANY,
 			PW_STREAM_FLAG_AUTOCONNECT |
 			PW_STREAM_FLAG_MAP_BUFFERS |
 			PW_STREAM_FLAG_RT_PROCESS,
-			params, n_params)) < 0)
+			params, n_params)) < 0) {
+		spa_pod_dynamic_builder_clean(&b);
 		return res;
+	}
+
+	spa_pod_dynamic_builder_clean(&b);
 
 	impl->rec_ringsize = sizeof(float) * impl->max_buffer_size * impl->info.rate / 1000;
 	impl->play_ringsize = sizeof(float) * ((impl->max_buffer_size * impl->info.rate / 1000) + impl->buffer_delay);
@@ -760,8 +911,13 @@ static void core_error(void *data, uint32_t id, int seq, int res, const char *me
 {
 	struct impl *impl = data;
 
-	pw_log_error("error id:%u seq:%d res:%d (%s): %s",
-			id, seq, res, spa_strerror(res), message);
+	if (res == -ENOENT) {
+		pw_log_info("id:%u seq:%d res:%d (%s): %s",
+				id, seq, res, spa_strerror(res), message);
+	} else {
+		pw_log_warn("error id:%u seq:%d res:%d (%s): %s",
+				id, seq, res, spa_strerror(res), message);
+	}
 
 	if (id == PW_ID_CORE && res == -EPIPE)
 		pw_impl_module_schedule_destroy(impl->module);
@@ -799,7 +955,9 @@ static void impl_destroy(struct impl *impl)
 		pw_core_disconnect(impl->core);
 	if (impl->spa_handle)
 		spa_plugin_loader_unload(impl->loader, impl->spa_handle);
+	pw_properties_free(impl->capture_props);
 	pw_properties_free(impl->source_props);
+	pw_properties_free(impl->playback_props);
 	pw_properties_free(impl->sink_props);
 
 	for (i = 0; i < impl->info.channels; i++) {
@@ -859,17 +1017,27 @@ static void parse_audio_info(struct pw_properties *props, struct spa_audio_info_
 	*info = SPA_AUDIO_INFO_RAW_INIT(
 			.format = SPA_AUDIO_FORMAT_F32P);
 	info->rate = pw_properties_get_uint32(props, PW_KEY_AUDIO_RATE, info->rate);
+	if (info->rate == 0)
+		info->rate = DEFAULT_RATE;
+
 	info->channels = pw_properties_get_uint32(props, PW_KEY_AUDIO_CHANNELS, info->channels);
+	info->channels = SPA_MIN(info->channels, SPA_AUDIO_MAX_CHANNELS);
 	if ((str = pw_properties_get(props, SPA_KEY_AUDIO_POSITION)) != NULL)
 		parse_position(info, str, strlen(str));
+	if (info->channels == 0)
+		parse_position(info, DEFAULT_POSITION, strlen(DEFAULT_POSITION));
 }
 
 static void copy_props(struct impl *impl, struct pw_properties *props, const char *key)
 {
 	const char *str;
 	if ((str = pw_properties_get(props, key)) != NULL) {
+		if (pw_properties_get(impl->capture_props, key) == NULL)
+			pw_properties_set(impl->capture_props, key, str);
 		if (pw_properties_get(impl->source_props, key) == NULL)
 			pw_properties_set(impl->source_props, key, str);
+		if (pw_properties_get(impl->playback_props, key) == NULL)
+			pw_properties_set(impl->playback_props, key, str);
 		if (pw_properties_get(impl->sink_props, key) == NULL)
 			pw_properties_set(impl->sink_props, key, str);
 	}
@@ -907,14 +1075,21 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 		goto error;
 	}
 
+	impl->capture_props = pw_properties_new(NULL, NULL);
 	impl->source_props = pw_properties_new(NULL, NULL);
+	impl->playback_props = pw_properties_new(NULL, NULL);
 	impl->sink_props = pw_properties_new(NULL, NULL);
-	if (impl->source_props == NULL || impl->sink_props == NULL) {
+	if (impl->source_props == NULL || impl->sink_props == NULL ||
+	    impl->capture_props == NULL || impl->playback_props == NULL) {
 		res = -errno;
 		pw_log_error( "can't create properties: %m");
 		goto error;
 	}
 
+	impl->monitor_mode = false;
+	if ((str = pw_properties_get(props, "monitor.mode")) != NULL)
+		impl->monitor_mode = pw_properties_parse_bool(str);
+
 	impl->module = module;
 	impl->context = context;
 
@@ -924,21 +1099,26 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 		pw_properties_setf(props, PW_KEY_NODE_LINK_GROUP, "echo-cancel-%u-%u", pid, id);
 	if (pw_properties_get(props, PW_KEY_NODE_VIRTUAL) == NULL)
 		pw_properties_set(props, PW_KEY_NODE_VIRTUAL, "true");
+	if (pw_properties_get(props, "resample.prefill") == NULL)
+		pw_properties_set(props, "resample.prefill", "true");
 
 	parse_audio_info(props, &impl->info);
 
-	if (impl->info.channels == 0) {
-		impl->info.channels = 2;
-		impl->info.position[0] = SPA_AUDIO_CHANNEL_FL;
-		impl->info.position[1] = SPA_AUDIO_CHANNEL_FR;
-	}
-	if (impl->info.rate == 0)
-		impl->info.rate = 48000;
-
+	if ((str = pw_properties_get(props, "capture.props")) != NULL)
+		pw_properties_update_string(impl->capture_props, str, strlen(str));
 	if ((str = pw_properties_get(props, "source.props")) != NULL)
 		pw_properties_update_string(impl->source_props, str, strlen(str));
 	if ((str = pw_properties_get(props, "sink.props")) != NULL)
 		pw_properties_update_string(impl->sink_props, str, strlen(str));
+	if ((str = pw_properties_get(props, "playback.props")) != NULL)
+		pw_properties_update_string(impl->playback_props, str, strlen(str));
+
+	if (pw_properties_get(impl->capture_props, PW_KEY_NODE_NAME) == NULL)
+		pw_properties_set(impl->capture_props, PW_KEY_NODE_NAME, "echo-cancel-capture");
+	if (pw_properties_get(impl->capture_props, PW_KEY_NODE_DESCRIPTION) == NULL)
+		pw_properties_set(impl->capture_props, PW_KEY_NODE_DESCRIPTION, "Echo-Cancel Capture");
+	if (pw_properties_get(impl->capture_props, PW_KEY_NODE_PASSIVE) == NULL)
+		pw_properties_set(impl->capture_props, PW_KEY_NODE_PASSIVE, "true");
 
 	if (pw_properties_get(impl->source_props, PW_KEY_NODE_NAME) == NULL)
 		pw_properties_set(impl->source_props, PW_KEY_NODE_NAME, "echo-cancel-source");
@@ -947,12 +1127,28 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	if (pw_properties_get(impl->source_props, PW_KEY_MEDIA_CLASS) == NULL)
 		pw_properties_set(impl->source_props, PW_KEY_MEDIA_CLASS, "Audio/Source");
 
+	if (pw_properties_get(impl->playback_props, PW_KEY_NODE_NAME) == NULL)
+		pw_properties_set(impl->playback_props, PW_KEY_NODE_NAME, "echo-cancel-playback");
+	if (pw_properties_get(impl->playback_props, PW_KEY_NODE_DESCRIPTION) == NULL)
+		pw_properties_set(impl->playback_props, PW_KEY_NODE_DESCRIPTION, "Echo-Cancel Playback");
+	if (pw_properties_get(impl->playback_props, PW_KEY_NODE_PASSIVE) == NULL)
+		pw_properties_set(impl->playback_props, PW_KEY_NODE_PASSIVE, "true");
+
 	if (pw_properties_get(impl->sink_props, PW_KEY_NODE_NAME) == NULL)
 		pw_properties_set(impl->sink_props, PW_KEY_NODE_NAME, "echo-cancel-sink");
 	if (pw_properties_get(impl->sink_props, PW_KEY_NODE_DESCRIPTION) == NULL)
 		pw_properties_set(impl->sink_props, PW_KEY_NODE_DESCRIPTION, "Echo-Cancel Sink");
 	if (pw_properties_get(impl->sink_props, PW_KEY_MEDIA_CLASS) == NULL)
-		pw_properties_set(impl->sink_props, PW_KEY_MEDIA_CLASS, "Audio/Sink");
+		pw_properties_set(impl->sink_props, PW_KEY_MEDIA_CLASS,
+				impl->monitor_mode ? "Stream/Input/Audio" : "Audio/Sink");
+	if (impl->monitor_mode) {
+		if (pw_properties_get(impl->sink_props, PW_KEY_NODE_PASSIVE) == NULL)
+			pw_properties_set(impl->sink_props, PW_KEY_NODE_PASSIVE, "true");
+		if (pw_properties_get(impl->sink_props, PW_KEY_STREAM_MONITOR) == NULL)
+			pw_properties_set(impl->sink_props, PW_KEY_STREAM_MONITOR, "true");
+		if (pw_properties_get(impl->sink_props, PW_KEY_STREAM_CAPTURE_SINK) == NULL)
+			pw_properties_set(impl->sink_props, PW_KEY_STREAM_CAPTURE_SINK, "true");
+	}
 
 	if ((str = pw_properties_get(props, "aec.method")) != NULL)
 		pw_log_warn("aec.method is not supported anymore use library.name");
@@ -974,7 +1170,7 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 
 	handle = spa_plugin_loader_load(impl->loader, SPA_NAME_AEC, &info);
 	if (handle == NULL) {
-		pw_log_error("AEC codec plugin %s not available library.name %s", SPA_NAME_AEC, path);
+		pw_log_error("aec plugin %s not available library.name %s", SPA_NAME_AEC, path);
 		return -ENOENT;
 	}
 
@@ -984,16 +1180,16 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	}
 	impl->aec = iface;
 	impl->spa_handle = handle;
-	if (impl->aec->iface.version != SPA_VERSION_AUDIO_AEC) {
-		pw_log_error("codec plugin %s has incompatible ABI version (%d != %d)",
+
+	if (impl->aec->iface.version > SPA_VERSION_AUDIO_AEC) {
+		pw_log_error("codec plugin %s has incompatible ABI version (%d > %d)",
 			SPA_NAME_AEC, impl->aec->iface.version, SPA_VERSION_AUDIO_AEC);
 		res = -ENOENT;
 		goto error;
 	}
 
-	(void)SPA_SUPPORT_INIT(SPA_TYPE_INTERFACE_AUDIO_AEC, (struct spa_audio_aec *)impl->aec);
-
-	pw_log_info("Using plugin AEC %s", impl->aec->name);
+	pw_log_info("Using plugin AEC %s with version %d", impl->aec->name,
+			impl->aec->iface.version);
 
 	if ((str = pw_properties_get(props, "aec.args")) != NULL)
 		aec_props = pw_properties_new_string(str);
@@ -1005,7 +1201,7 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	pw_properties_free(aec_props);
 
 	if (res < 0) {
-		pw_log_error("codec plugin %s create failed: %s", impl->aec->name,
+		pw_log_error("aec plugin %s create failed: %s", impl->aec->name,
 				spa_strerror(res));
 		goto error;
 	}
@@ -1059,6 +1255,7 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	copy_props(impl, props, PW_KEY_NODE_LATENCY);
 	copy_props(impl, props, SPA_KEY_AUDIO_CHANNELS);
 	copy_props(impl, props, SPA_KEY_AUDIO_POSITION);
+	copy_props(impl, props, "resample.prefill");
 
 	impl->max_buffer_size = pw_properties_get_uint32(props,"buffer.max_size", MAX_BUFSIZE_MS);
 
