@@ -38,6 +38,7 @@
 #include <spa/utils/result.h>
 #include <spa/utils/string.h>
 #include <spa/utils/json.h>
+#include <spa/support/cpu.h>
 #include <spa/param/profiler.h>
 #include <spa/pod/dynamic.h>
 #include <spa/debug/pod.h>
@@ -505,6 +506,7 @@ struct node {
 
 	unsigned int n_deps;
 	unsigned int visited:1;
+	unsigned int disabled:1;
 };
 
 struct link {
@@ -521,6 +523,7 @@ struct graph_port {
 	const struct fc_descriptor *desc;
 	void **hndl;
 	uint32_t port;
+	unsigned next:1;
 };
 
 struct graph_hndl {
@@ -558,6 +561,8 @@ struct impl {
 	struct spa_hook core_proxy_listener;
 	struct spa_hook core_listener;
 
+	struct dsp_ops dsp;
+
 	struct spa_list plugin_list;
 
 	struct pw_properties *capture_props;
@@ -599,7 +604,7 @@ static void playback_process(void *d)
 	struct impl *impl = d;
 	struct pw_buffer *in, *out;
 	struct graph *graph = &impl->graph;
-	uint32_t i, insize = 0, outsize = 0, n_hndl = graph->n_hndl;
+	uint32_t i, j, insize = 0, outsize = 0, n_hndl = graph->n_hndl;
 	int32_t stride = 0;
 	struct graph_port *port;
 	struct spa_data *bd;
@@ -613,7 +618,7 @@ static void playback_process(void *d)
 	if (in == NULL || out == NULL)
 		goto done;
 
-	for (i = 0; i < in->buffer->n_datas; i++) {
+	for (i = 0, j = 0; i < in->buffer->n_datas; i++) {
 		uint32_t offs, size;
 
 		bd = &in->buffer->datas[i];
@@ -621,12 +626,15 @@ static void playback_process(void *d)
 		offs = SPA_MIN(bd->chunk->offset, bd->maxsize);
 		size = SPA_MIN(bd->chunk->size, bd->maxsize - offs);
 
-		port = i < graph->n_input ? &graph->input[i] : NULL;
-
-		if (port && port->desc)
-			port->desc->connect_port(*port->hndl, port->port,
-				SPA_PTROFF(bd->data, offs, void));
+		while (j < graph->n_input) {
+			port = &graph->input[j++];
+			if (port->desc)
+				port->desc->connect_port(*port->hndl, port->port,
+					SPA_PTROFF(bd->data, offs, void));
+			if (!port->next)
+				break;
 
+		}
 		insize = i == 0 ? size : SPA_MIN(insize, size);
 		stride = SPA_MAX(stride, bd->chunk->stride);
 	}
@@ -1063,9 +1071,9 @@ static const struct pw_stream_events out_stream_events = {
 static int setup_streams(struct impl *impl)
 {
 	int res;
-	uint32_t i, n_params;
-	uint32_t offsets[512];
-	const struct spa_pod *params[512];
+	uint32_t i, n_params, *offs;
+	struct pw_array offsets;
+	const struct spa_pod **params = NULL;
 	struct spa_pod_dynamic_builder b;
 	struct graph *graph = &impl->graph;
 
@@ -1089,23 +1097,40 @@ static int setup_streams(struct impl *impl)
 			&impl->playback_listener,
 			&out_stream_events, impl);
 
-	n_params = 0;
 	spa_pod_dynamic_builder_init(&b, NULL, 0, 4096);
+	pw_array_init(&offsets, 512);
 
-	offsets[n_params++] = b.b.state.offset;
+	if ((offs = pw_array_add(&offsets, sizeof(uint32_t))) == NULL) {
+		res = -errno;
+		goto done;
+	}
+	*offs = b.b.state.offset;
 	spa_format_audio_raw_build(&b.b,
 			SPA_PARAM_EnumFormat, &impl->capture_info);
 
 	for (i = 0; i < graph->n_control; i++) {
-		offsets[n_params++] = b.b.state.offset;
+		if ((offs = pw_array_add(&offsets, sizeof(uint32_t))) != NULL)
+			*offs = b.b.state.offset;
 		get_prop_info(graph, &b.b, i);
 	}
 
-	offsets[n_params++] = b.b.state.offset;
+	if ((offs = pw_array_add(&offsets, sizeof(uint32_t))) != NULL)
+		*offs = b.b.state.offset;
 	get_props_param(graph, &b.b);
 
+	n_params = pw_array_get_len(&offsets, uint32_t);
+	if (n_params == 0) {
+		res = -ENOMEM;
+		goto done;
+	}
+	if ((params = calloc(n_params, sizeof(struct spa_pod*))) == NULL) {
+		res = -errno;
+		goto done;
+	}
+
+	offs = offsets.data;
 	for (i = 0; i < n_params; i++)
-		params[i] = spa_pod_builder_deref(&b.b, offsets[i]);
+		params[i] = spa_pod_builder_deref(&b.b, offs[i]);
 
 	res = pw_stream_connect(impl->capture,
 			PW_DIRECTION_INPUT,
@@ -1117,7 +1142,7 @@ static int setup_streams(struct impl *impl)
 
 	spa_pod_dynamic_builder_clean(&b);
 	if (res < 0)
-		return res;
+		goto done;
 
 	n_params = 0;
 	spa_pod_dynamic_builder_init(&b, NULL, 0, 4096);
@@ -1134,11 +1159,11 @@ static int setup_streams(struct impl *impl)
 			params, n_params);
 	spa_pod_dynamic_builder_clean(&b);
 
-	if (res < 0)
-		return res;
+done:
+	free(params);
+	pw_array_clear(&offsets);
 
-
-	return 0;
+	return res < 0 ? res : 0;
 }
 
 static uint32_t count_array(struct spa_json *json)
@@ -1179,14 +1204,14 @@ static struct plugin *plugin_load(struct impl *impl, const char *type, const cha
 	support = pw_context_get_support(impl->context, &n_support);
 
 	if (spa_streq(type, "builtin")) {
-		pl = load_builtin_plugin(support, n_support, path, NULL);
+		pl = load_builtin_plugin(support, n_support, &impl->dsp, path, NULL);
 	}
 	else if (spa_streq(type, "ladspa")) {
-		pl = load_ladspa_plugin(support, n_support, path, NULL);
+		pl = load_ladspa_plugin(support, n_support, &impl->dsp, path, NULL);
 	}
 	else if (spa_streq(type, "lv2")) {
 #ifdef HAVE_LILV
-		pl = load_lv2_plugin(support, n_support, path, NULL);
+		pl = load_lv2_plugin(support, n_support, &impl->dsp, path, NULL);
 #else
 		pw_log_error("filter-chain is compiled without lv2 support");
 		pl = NULL;
@@ -1849,7 +1874,7 @@ static int setup_graph(struct graph *graph, struct spa_json *inputs, struct spa_
 		n_nodes++;
 	}
 	graph->n_input = 0;
-	graph->input = calloc(n_input * n_hndl, sizeof(struct graph_port));
+	graph->input = calloc(n_input * 16 * n_hndl, sizeof(struct graph_port));
 	graph->n_output = 0;
 	graph->output = calloc(n_output * n_hndl, sizeof(struct graph_port));
 
@@ -1869,8 +1894,8 @@ static int setup_graph(struct graph *graph, struct spa_json *inputs, struct spa_
 		} else {
 			struct spa_json it = *inputs;
 			while (spa_json_get_string(&it, v, sizeof(v)) > 0) {
-				gp = &graph->input[graph->n_input];
 				if (spa_streq(v, "null")) {
+					gp = &graph->input[graph->n_input++];
 					gp->desc = NULL;
 					pw_log_info("ignore input port %d", graph->n_input);
 				} else if ((port = find_port(first, v, FC_PORT_INPUT)) == NULL) {
@@ -1893,14 +1918,41 @@ static int setup_graph(struct graph *graph, struct spa_json *inputs, struct spa_
 						res = -EBUSY;
 						goto error;
 					}
-					pw_log_info("input port %s[%d]:%s",
+
+					if (d->flags & FC_DESCRIPTOR_COPY) {
+						for (j = 0; j < desc->n_output; j++) {
+							struct port *p = &port->node->output_port[j];
+							struct link *link;
+
+							gp = NULL;
+							spa_list_for_each(link, &p->link_list, output_link) {
+								struct port *peer = link->input;
+
+								pw_log_info("copy input port %s[%d]:%s",
+									port->node->name, i,
+									d->ports[port->p].name);
+								peer->external = graph->n_input;
+								gp = &graph->input[graph->n_input++];
+								gp->desc = peer->node->desc->desc;
+								gp->hndl = &peer->node->hndl[i];
+								gp->port = peer->p;
+								gp->next = true;
+							}
+							if (gp != NULL)
+								gp->next = false;
+						}
+						port->node->disabled = true;
+					} else {
+						pw_log_info("input port %s[%d]:%s",
 							port->node->name, i, d->ports[port->p].name);
-					port->external = graph->n_input;
-					gp->desc = d;
-					gp->hndl = &port->node->hndl[i];
-					gp->port = port->p;
+						port->external = graph->n_input;
+						gp = &graph->input[graph->n_input++];
+						gp->desc = d;
+						gp->hndl = &port->node->hndl[i];
+						gp->port = port->p;
+						gp->next = false;
+					}
 				}
-				graph->n_input++;
 			}
 		}
 		if (outputs == NULL) {
@@ -1965,11 +2017,12 @@ static int setup_graph(struct graph *graph, struct spa_json *inputs, struct spa_
 		desc = node->desc;
 		d = desc->desc;
 
-		for (i = 0; i < n_hndl; i++) {
-			gh = &graph->hndl[graph->n_hndl++];
-			gh->hndl = &node->hndl[i];
-			gh->desc = d;
-
+		if (!node->disabled) {
+			for (i = 0; i < n_hndl; i++) {
+				gh = &graph->hndl[graph->n_hndl++];
+				gh->hndl = &node->hndl[i];
+				gh->desc = d;
+			}
 		}
 		for (i = 0; i < desc->n_output; i++) {
 			spa_list_for_each(link, &node->output_port[i].link_list, output_link)
@@ -2088,8 +2141,13 @@ static void core_error(void *data, uint32_t id, int seq, int res, const char *me
 {
 	struct impl *impl = data;
 
-	pw_log_error("error id:%u seq:%d res:%d (%s): %s",
-			id, seq, res, spa_strerror(res), message);
+	if (res == -ENOENT) {
+		pw_log_info("message id:%u seq:%d res:%d (%s): %s",
+				id, seq, res, spa_strerror(res), message);
+	} else {
+		pw_log_warn("error id:%u seq:%d res:%d (%s): %s",
+				id, seq, res, spa_strerror(res), message);
+	}
 
 	if (id == PW_ID_CORE && res == -EPIPE)
 		pw_impl_module_schedule_destroy(impl->module);
@@ -2206,6 +2264,9 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	uint32_t pid = getpid();
 	const char *str;
 	int res;
+	const struct spa_support *support;
+	uint32_t n_support;
+	struct spa_cpu *cpu_iface;
 
 	PW_LOG_TOPIC_INIT(mod_topic);
 
@@ -2236,10 +2297,16 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 
 	impl->module = module;
 	impl->context = context;
-
 	impl->graph.impl = impl;
+
 	spa_list_init(&impl->plugin_list);
 
+	support = pw_context_get_support(impl->context, &n_support);
+
+	cpu_iface = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_CPU);
+	impl->dsp.cpu_flags = cpu_iface ? spa_cpu_get_flags(cpu_iface) : 0;
+	dsp_ops_init(&impl->dsp);
+
 	if (pw_properties_get(props, PW_KEY_NODE_GROUP) == NULL)
 		pw_properties_setf(props, PW_KEY_NODE_GROUP, "filter-chain-%u-%u", pid, id);
 	if (pw_properties_get(props, PW_KEY_NODE_LINK_GROUP) == NULL)
