@@ -24,12 +24,17 @@
 
 #include "config.h"
 
+#include <float.h>
 #include <math.h>
 #ifdef HAVE_SNDFILE
 #include <sndfile.h>
 #endif
 
 #include <spa/utils/json.h>
+#include <spa/utils/result.h>
+#include <spa/support/cpu.h>
+#include <spa/plugins/audioconvert/resample.h>
+
 #include <pipewire/log.h>
 
 #include "plugin.h"
@@ -37,6 +42,9 @@
 #include "biquad.h"
 #include "pffft.h"
 #include "convolver.h"
+#include "dsp-ops.h"
+
+static struct dsp_ops *dsp_ops;
 
 struct builtin {
 	unsigned long rate;
@@ -49,7 +57,7 @@ struct builtin {
 };
 
 static void *builtin_instantiate(const struct fc_descriptor * Descriptor,
-		unsigned long *SampleRate, int index, const char *config)
+		unsigned long SampleRate, int index, const char *config)
 {
 	struct builtin *impl;
 
@@ -57,7 +65,7 @@ static void *builtin_instantiate(const struct fc_descriptor * Descriptor,
 	if (impl == NULL)
 		return NULL;
 
-	impl->rate = *SampleRate;
+	impl->rate = SampleRate;
 
 	return impl;
 }
@@ -79,7 +87,7 @@ static void copy_run(void * Instance, unsigned long SampleCount)
 {
 	struct builtin *impl = Instance;
 	float *in = impl->port[1], *out = impl->port[0];
-	memcpy(out, in, SampleCount * sizeof(float));
+	dsp_ops_copy(dsp_ops, out, in, SampleCount);
 }
 
 static struct fc_port copy_ports[] = {
@@ -95,6 +103,7 @@ static struct fc_port copy_ports[] = {
 
 static const struct fc_descriptor copy_desc = {
 	.name = "copy",
+	.flags = FC_DESCRIPTOR_COPY,
 
 	.n_ports = 2,
 	.ports = copy_ports,
@@ -109,10 +118,10 @@ static const struct fc_descriptor copy_desc = {
 static void mixer_run(void * Instance, unsigned long SampleCount)
 {
 	struct builtin *impl = Instance;
-	int i;
-	unsigned long j;
+	int i, n_src = 0;
 	float *out = impl->port[0];
-	bool first = true;
+	const void *src[8];
+	float gains[8];
 
 	if (out == NULL)
 		return;
@@ -124,24 +133,10 @@ static void mixer_run(void * Instance, unsigned long SampleCount)
 		if (in == NULL || gain == 0.0f)
 			continue;
 
-		if (first) {
-			if (gain == 1.0f)
-				memcpy(out, in, SampleCount * sizeof(float));
-			else
-				for (j = 0; j < SampleCount; j++)
-					out[j] = in[j] * gain;
-			first = false;
-		} else {
-			if (gain == 1.0f)
-				for (j = 0; j < SampleCount; j++)
-					out[j] += in[j];
-			else
-				for (j = 0; j < SampleCount; j++)
-					out[j] += in[j] * gain;
-		}
+		src[n_src] = in;
+		gains[n_src++] = gain;
 	}
-	if (first)
-		memset(out, 0, SampleCount * sizeof(float));
+	dsp_ops_mix_gain(dsp_ops, out, src, gains, n_src, SampleCount);
 }
 
 static struct fc_port mixer_ports[] = {
@@ -261,21 +256,18 @@ static struct fc_port bq_ports[] = {
 	{ .index = 4,
 	  .name = "Gain",
 	  .flags = FC_PORT_INPUT | FC_PORT_CONTROL,
-	  .def = 0.0f, .min = -120.0f, .max = 5.0f,
+	  .def = 0.0f, .min = -120.0f, .max = 20.0f,
 	},
 };
 
 static void bq_run(struct builtin *impl, unsigned long samples, int type)
 {
 	struct biquad *bq = &impl->bq;
-	unsigned long i;
 	float *out = impl->port[0];
 	float *in = impl->port[1];
 	float freq = impl->port[2][0];
 	float Q = impl->port[3][0];
 	float gain = impl->port[4][0];
-	float x1, x2, y1, y2;
-	float b0, b1, b2, a1, a2;
 
 	if (impl->freq != freq || impl->Q != Q || impl->gain != gain) {
 		impl->freq = freq;
@@ -283,28 +275,7 @@ static void bq_run(struct builtin *impl, unsigned long samples, int type)
 		impl->gain = gain;
 		biquad_set(bq, type, freq * 2 / impl->rate, Q, gain);
 	}
-	x1 = bq->x1;
-	x2 = bq->x2;
-	y1 = bq->y1;
-	y2 = bq->y2;
-	b0 = bq->b0;
-	b1 = bq->b1;
-	b2 = bq->b2;
-	a1 = bq->a1;
-	a2 = bq->a2;
-	for (i = 0; i < samples; i++) {
-		float x = in[i];
-		float y = b0 * x + b1 * x1 + b2 * x2 - a1 * y1 - a2 * y2;
-		out[i] = y;
-		x2 = x1;
-		x1 = x;
-		y2 = y1;
-		y1 = y;
-	}
-	bq->x1 = x1;
-	bq->x2 = x2;
-	bq->y1 = y1;
-	bq->y2 = y2;
+	dsp_ops_biquad_run(dsp_ops, bq, out, in, samples);
 }
 
 /** bq_lowpass */
@@ -480,7 +451,7 @@ static float *read_samples(const char *filename, float gain, int delay, int offs
 	spa_zero(info);
 	f = sf_open(filename, SFM_READ, &info) ;
 	if (f == NULL) {
-		fprintf(stderr, "can't open %s", filename);
+		pw_log_error("can't open %s", filename);
 		return NULL;
 	}
 
@@ -570,8 +541,73 @@ static float *create_dirac(const char *filename, float gain, int delay, int offs
 	return samples;
 }
 
+static float *resample_buffer(float *samples, int *n_samples,
+		unsigned long in_rate, unsigned long out_rate, uint32_t quality)
+{
+	uint32_t in_len, out_len, total_out = 0;
+	int out_n_samples;
+	float *out_samples, *out_buf, *in_buf;
+	struct resample r;
+	int res;
+
+	spa_zero(r);
+	r.channels = 1;
+	r.i_rate = in_rate;
+	r.o_rate = out_rate;
+	r.cpu_flags = dsp_ops->cpu_flags;
+	r.quality = quality;
+	if ((res = resample_native_init(&r)) < 0) {
+		pw_log_error("resampling failed: %s", spa_strerror(res));
+		errno = -res;
+		return NULL;
+	}
+
+	out_n_samples = SPA_ROUND_UP(*n_samples * out_rate, in_rate) / in_rate;
+	out_samples = calloc(out_n_samples, sizeof(float));
+	if (out_samples == NULL)
+		goto error;
+
+	in_len = *n_samples;
+	in_buf = samples;
+	out_len = out_n_samples;
+	out_buf = out_samples;
+
+	pw_log_info("Resampling filter: rate: %lu => %lu, n_samples: %u => %u, q:%u",
+		    in_rate, out_rate, in_len, out_len, quality);
+
+	resample_process(&r, (void*)&in_buf, &in_len, (void*)&out_buf, &out_len);
+	pw_log_debug("resampled: %u -> %u samples", in_len, out_len);
+	total_out += out_len;
+
+	in_len = resample_delay(&r);
+	in_buf = calloc(in_len, sizeof(float));
+	if (in_buf == NULL)
+		goto error;
+
+	out_buf = out_samples + total_out;
+	out_len = out_n_samples - total_out;
+
+	pw_log_debug("flushing resampler: %u in %u out", in_len, out_len);
+	resample_process(&r, (void*)&in_buf, &in_len, (void*)&out_buf, &out_len);
+	pw_log_debug("flushed: %u -> %u samples", in_len, out_len);
+	total_out += out_len;
+
+	free(in_buf);
+	free(samples);
+	resample_free(&r);
+
+	*n_samples = total_out;
+	return out_samples;
+
+error:
+	resample_free(&r);
+	free(samples);
+	free(out_samples);
+	return NULL;
+}
+
 static void * convolver_instantiate(const struct fc_descriptor * Descriptor,
-		unsigned long *SampleRate, int index, const char *config)
+		unsigned long SampleRate, int index, const char *config)
 {
 	struct convolver_impl *impl;
 	float *samples;
@@ -582,8 +618,11 @@ static void * convolver_instantiate(const struct fc_descriptor * Descriptor,
 	char filename[PATH_MAX] = "";
 	int blocksize = 0, tailsize = 0;
 	int delay = 0;
+	int resample_quality = RESAMPLE_DEFAULT_QUALITY;
 	float gain = 1.0f;
+	unsigned long rate;
 
+	errno = EINVAL;
 	if (config == NULL)
 		return NULL;
 
@@ -593,42 +632,66 @@ static void * convolver_instantiate(const struct fc_descriptor * Descriptor,
 
 	while (spa_json_get_string(&it[1], key, sizeof(key)) > 0) {
 		if (spa_streq(key, "blocksize")) {
-			if (spa_json_get_int(&it[1], &blocksize) <= 0)
+			if (spa_json_get_int(&it[1], &blocksize) <= 0) {
+				pw_log_error("convolver:blocksize requires a number");
 				return NULL;
+			}
 		}
 		else if (spa_streq(key, "tailsize")) {
-			if (spa_json_get_int(&it[1], &tailsize) <= 0)
+			if (spa_json_get_int(&it[1], &tailsize) <= 0) {
+				pw_log_error("convolver:tailsize requires a number");
 				return NULL;
+			}
 		}
 		else if (spa_streq(key, "gain")) {
-			if (spa_json_get_float(&it[1], &gain) <= 0)
+			if (spa_json_get_float(&it[1], &gain) <= 0) {
+				pw_log_error("convolver:gain requires a number");
 				return NULL;
+			}
 		}
 		else if (spa_streq(key, "delay")) {
-			if (spa_json_get_int(&it[1], &delay) <= 0)
+			if (spa_json_get_int(&it[1], &delay) <= 0) {
+				pw_log_error("convolver:delay requires a number");
 				return NULL;
+			}
 		}
 		else if (spa_streq(key, "filename")) {
-			if (spa_json_get_string(&it[1], filename, sizeof(filename)) <= 0)
+			if (spa_json_get_string(&it[1], filename, sizeof(filename)) <= 0) {
+				pw_log_error("convolver:filename requires a string");
 				return NULL;
+			}
 		}
 		else if (spa_streq(key, "offset")) {
-			if (spa_json_get_int(&it[1], &offset) <= 0)
+			if (spa_json_get_int(&it[1], &offset) <= 0) {
+				pw_log_error("convolver:offset requires a number");
 				return NULL;
+			}
 		}
 		else if (spa_streq(key, "length")) {
-			if (spa_json_get_int(&it[1], &length) <= 0)
+			if (spa_json_get_int(&it[1], &length) <= 0) {
+				pw_log_error("convolver:length requires a number");
 				return NULL;
+			}
 		}
 		else if (spa_streq(key, "channel")) {
-			if (spa_json_get_int(&it[1], &channel) <= 0)
+			if (spa_json_get_int(&it[1], &channel) <= 0) {
+				pw_log_error("convolver:channel requires a number");
 				return NULL;
+			}
+		}
+		else if (spa_streq(key, "resample_quality")) {
+			if (spa_json_get_int(&it[1], &resample_quality) <= 0) {
+				pw_log_error("convolver:resample_quality requires a number");
+				return NULL;
+			}
 		}
 		else if (spa_json_next(&it[1], &val) < 0)
 			break;
 	}
-	if (!filename[0])
+	if (!filename[0]) {
+		pw_log_error("convolver:filename was not given");
 		return NULL;
+	}
 
 	if (delay < 0)
 		delay = 0;
@@ -642,26 +705,33 @@ static void * convolver_instantiate(const struct fc_descriptor * Descriptor,
 		samples = create_dirac(filename, gain, delay, offset,
 				length, &n_samples);
 	} else {
+		rate = SampleRate;
 		samples = read_samples(filename, gain, delay, offset,
-				length, channel, SampleRate, &n_samples);
+				length, channel, &rate, &n_samples);
+		if (rate != SampleRate)
+			samples = resample_buffer(samples, &n_samples,
+					rate, SampleRate, resample_quality);
 	}
-	if (samples == NULL)
+	if (samples == NULL) {
+		errno = ENOENT;
 		return NULL;
+	}
 
 	if (blocksize <= 0)
 		blocksize = SPA_CLAMP(n_samples, 64, 256);
 	if (tailsize <= 0)
-		tailsize = SPA_CLAMP(4096, blocksize, 4096);
+		tailsize = SPA_CLAMP(4096, blocksize, 32768);
 
-	pw_log_info("using %d:%d blocksize ir:%s", blocksize, tailsize, filename);
+	pw_log_info("using n_samples:%u %d:%d blocksize ir:%s", n_samples,
+			blocksize, tailsize, filename);
 
 	impl = calloc(1, sizeof(*impl));
 	if (impl == NULL)
 		goto error;
 
-	impl->rate = *SampleRate;
+	impl->rate = SampleRate;
 
-	impl->conv = convolver_new(blocksize, tailsize, samples, n_samples);
+	impl->conv = convolver_new(dsp_ops, blocksize, tailsize, samples, n_samples);
 	if (impl->conv == NULL)
 		goto error;
 
@@ -725,6 +795,137 @@ static const struct fc_descriptor convolve_desc = {
 	.cleanup = convolver_cleanup,
 };
 
+/** delay */
+struct delay_impl {
+	unsigned long rate;
+	float *port[4];
+
+	float delay;
+	uint32_t delay_samples;
+	uint32_t buffer_samples;
+	float *buffer;
+	uint32_t ptr;
+};
+
+static void delay_cleanup(void * Instance)
+{
+	struct delay_impl *impl = Instance;
+	free(impl->buffer);
+	free(impl);
+}
+
+static void *delay_instantiate(const struct fc_descriptor * Descriptor,
+		unsigned long SampleRate, int index, const char *config)
+{
+	struct delay_impl *impl;
+	struct spa_json it[2];
+	const char *val;
+	char key[256];
+	float max_delay = 1.0f;
+
+	if (config == NULL) {
+		errno = EINVAL;
+		return NULL;
+	}
+
+	spa_json_init(&it[0], config, strlen(config));
+	if (spa_json_enter_object(&it[0], &it[1]) <= 0)
+		return NULL;
+
+	while (spa_json_get_string(&it[1], key, sizeof(key)) > 0) {
+		if (spa_streq(key, "max-delay")) {
+			if (spa_json_get_float(&it[1], &max_delay) <= 0) {
+				pw_log_error("delay:max-delay requires a number");
+				return NULL;
+			}
+		}
+		else if (spa_json_next(&it[1], &val) < 0)
+			break;
+	}
+	if (max_delay <= 0.0f)
+		max_delay = 1.0f;
+
+	impl = calloc(1, sizeof(*impl));
+	if (impl == NULL)
+		return NULL;
+
+	impl->rate = SampleRate;
+	impl->buffer_samples = max_delay * impl->rate;
+	pw_log_info("max-delay:%f seconds rate:%lu samples:%d", max_delay, impl->rate, impl->buffer_samples);
+
+	impl->buffer = calloc(impl->buffer_samples, sizeof(float));
+	if (impl->buffer == NULL) {
+		delay_cleanup(impl);
+		return NULL;
+	}
+	return impl;
+}
+
+static void delay_connect_port(void * Instance, unsigned long Port,
+                        float * DataLocation)
+{
+	struct delay_impl *impl = Instance;
+	if (Port > 2)
+		return;
+	impl->port[Port] = DataLocation;
+}
+
+static void delay_run(void * Instance, unsigned long SampleCount)
+{
+	struct delay_impl *impl = Instance;
+	float *in = impl->port[1], *out = impl->port[0];
+	float delay = impl->port[2][0];
+	unsigned long n;
+	uint32_t r, w;
+
+	if (delay != impl->delay) {
+		impl->delay_samples = SPA_CLAMP(delay * impl->rate, 0, impl->buffer_samples-1);
+		impl->delay = delay;
+	}
+	r = impl->ptr;
+	w = impl->ptr + impl->delay_samples;
+	if (w >= impl->buffer_samples)
+		w -= impl->buffer_samples;
+
+	for (n = 0; n < SampleCount; n++) {
+		impl->buffer[w] = in[n];
+		out[n] = impl->buffer[r];
+		if (++r >= impl->buffer_samples)
+			r = 0;
+		if (++w >= impl->buffer_samples)
+			w = 0;
+	}
+	impl->ptr = r;
+}
+
+static struct fc_port delay_ports[] = {
+	{ .index = 0,
+	  .name = "Out",
+	  .flags = FC_PORT_OUTPUT | FC_PORT_AUDIO,
+	},
+	{ .index = 1,
+	  .name = "In",
+	  .flags = FC_PORT_INPUT | FC_PORT_AUDIO,
+	},
+	{ .index = 2,
+	  .name = "Delay (s)",
+	  .flags = FC_PORT_INPUT | FC_PORT_CONTROL,
+	  .def = 0.0f, .min = 0.0f, .max = 100.0f
+	},
+};
+
+static const struct fc_descriptor delay_desc = {
+	.name = "delay",
+
+	.n_ports = 3,
+	.ports = delay_ports,
+
+	.instantiate = delay_instantiate,
+	.connect_port = delay_connect_port,
+	.run = delay_run,
+	.cleanup = delay_cleanup,
+};
+
 static const struct fc_descriptor * builtin_descriptor(unsigned long Index)
 {
 	switch(Index) {
@@ -750,6 +951,8 @@ static const struct fc_descriptor * builtin_descriptor(unsigned long Index)
 		return &copy_desc;
 	case 10:
 		return &convolve_desc;
+	case 11:
+		return &delay_desc;
 	}
 	return NULL;
 }
@@ -771,12 +974,10 @@ static struct fc_plugin builtin_plugin = {
 	.make_desc = builtin_make_desc
 };
 
-struct fc_plugin *load_builtin_plugin(const char *plugin, const char *config)
+struct fc_plugin *load_builtin_plugin(const struct spa_support *support, uint32_t n_support,
+		struct dsp_ops *dsp, const char *plugin, const char *config)
 {
+	dsp_ops = dsp;
+	pffft_select_cpu(dsp->cpu_flags);
 	return &builtin_plugin;
 }
-
-void init_builtin_plugin(uint32_t cpu_flags)
-{
-	pffft_select_cpu(cpu_flags);
-}
