@@ -312,7 +312,7 @@ static void manager_added(void *d, struct pw_manager_object *o)
 	pw_properties_setf(props, PW_KEY_NODE_NAME,
 			"combine_output.sink-%u.%s", data->module->index, sink_name);
 	pw_properties_set(props, PW_KEY_NODE_DESCRIPTION, data->sink_name);
-	pw_properties_set(props, PW_KEY_NODE_TARGET, sink_name);
+	pw_properties_set(props, PW_KEY_TARGET_OBJECT, sink_name);
 	pw_properties_setf(props, PW_KEY_NODE_GROUP, "combine_sink-%u", data->module->index);
 	pw_properties_setf(props, PW_KEY_NODE_LINK_GROUP, "combine_sink-%u", data->module->index);
 	pw_properties_set(props, PW_KEY_NODE_DONT_RECONNECT, "true");
