@@ -30,7 +30,6 @@
 
 #include "../defs.h"
 #include "../module.h"
-#include "registry.h"
 
 #define NAME "ladspa-sink"
 
@@ -60,7 +59,7 @@ static const struct pw_impl_module_events module_events = {
 	.destroy = module_destroy
 };
 
-static int module_ladspa_sink_load(struct client *client, struct module *module)
+static int module_ladspa_sink_load(struct module *module)
 {
 	struct module_ladspa_sink_data *data = module->user_data;
 	FILE *f;
@@ -144,12 +143,6 @@ static int module_ladspa_sink_unload(struct module *module)
 	return 0;
 }
 
-static const struct module_methods module_ladspa_sink_methods = {
-	VERSION_MODULE_METHODS,
-	.load = module_ladspa_sink_load,
-	.unload = module_ladspa_sink_unload,
-};
-
 static const struct spa_dict_item module_ladspa_sink_info[] = {
 	{ PW_KEY_MODULE_AUTHOR, "Wim Taymans <wim.taymans@gmail.com>" },
 	{ PW_KEY_MODULE_DESCRIPTION, "Virtual LADSPA sink" },
@@ -184,11 +177,11 @@ static void position_to_props(struct spa_audio_info_raw *info, struct pw_propert
 	pw_properties_set(props, SPA_KEY_AUDIO_POSITION, s);
 }
 
-struct module *create_module_ladspa_sink(struct impl *impl, const char *argument)
+static int module_ladspa_sink_prepare(struct module * const module)
 {
-	struct module *module;
-	struct module_ladspa_sink_data *d;
-	struct pw_properties *props = NULL, *playback_props = NULL, *capture_props = NULL;
+	struct module_ladspa_sink_data * const d = module->user_data;
+	struct pw_properties * const props = module->props;
+	struct pw_properties *playback_props = NULL, *capture_props = NULL;
 	const char *str;
 	struct spa_audio_info_raw capture_info = { 0 };
 	struct spa_audio_info_raw playback_info = { 0 };
@@ -196,15 +189,12 @@ struct module *create_module_ladspa_sink(struct impl *impl, const char *argument
 
 	PW_LOG_TOPIC_INIT(mod_topic);
 
-	props = pw_properties_new_dict(&SPA_DICT_INIT_ARRAY(module_ladspa_sink_info));
 	capture_props = pw_properties_new(NULL, NULL);
 	playback_props = pw_properties_new(NULL, NULL);
-	if (!props || !capture_props || !playback_props) {
+	if (!capture_props || !playback_props) {
 		res = -EINVAL;
 		goto out;
 	}
-	if (argument)
-		module_args_add_props(props, argument);
 
 	if ((str = pw_properties_get(props, "sink_name")) != NULL) {
 		pw_properties_set(capture_props, PW_KEY_NODE_NAME, str);
@@ -229,11 +219,11 @@ struct module *create_module_ladspa_sink(struct impl *impl, const char *argument
 
 	if ((str = pw_properties_get(props, "master")) != NULL ||
 	    (str = pw_properties_get(props, "sink_master")) != NULL) {
-		pw_properties_set(playback_props, PW_KEY_NODE_TARGET, str);
+		pw_properties_set(playback_props, PW_KEY_TARGET_OBJECT, str);
 		pw_properties_set(props, "master", NULL);
 	}
 
-	if (module_args_to_audioinfo(impl, props, &capture_info) < 0) {
+	if (module_args_to_audioinfo(module->impl, props, &capture_info) < 0) {
 		res = -EINVAL;
 		goto out;
 	}
@@ -245,23 +235,23 @@ struct module *create_module_ladspa_sink(struct impl *impl, const char *argument
 	if (pw_properties_get(playback_props, PW_KEY_NODE_PASSIVE) == NULL)
 		pw_properties_set(playback_props, PW_KEY_NODE_PASSIVE, "true");
 
-	module = module_new(impl, &module_ladspa_sink_methods, sizeof(*d));
-	if (module == NULL) {
-		res = -errno;
-		goto out;
-	}
-
-	module->props = props;
-	d = module->user_data;
 	d->module = module;
 	d->capture_props = capture_props;
 	d->playback_props = playback_props;
 
-	return module;
+	return 0;
 out:
-	pw_properties_free(props);
 	pw_properties_free(playback_props);
 	pw_properties_free(capture_props);
-	errno = -res;
-	return NULL;
+
+	return res;
 }
+
+DEFINE_MODULE_INFO(module_ladspa_sink) = {
+	.name = "module-ladspa-sink",
+	.prepare = module_ladspa_sink_prepare,
+	.load = module_ladspa_sink_load,
+	.unload = module_ladspa_sink_unload,
+	.properties = &SPA_DICT_INIT_ARRAY(module_ladspa_sink_info),
+	.data_size = sizeof(struct module_ladspa_sink_data),
+};
