@@ -30,7 +30,6 @@
 
 #include "../defs.h"
 #include "../module.h"
-#include "registry.h"
 
 #define NAME "ladspa-source"
 
@@ -60,7 +59,7 @@ static const struct pw_impl_module_events module_events = {
 	.destroy = module_destroy
 };
 
-static int module_ladspa_source_load(struct client *client, struct module *module)
+static int module_ladspa_source_load(struct module *module)
 {
 	struct module_ladspa_source_data *data = module->user_data;
 	FILE *f;
@@ -75,8 +74,12 @@ static int module_ladspa_source_load(struct client *client, struct module *modul
 
 	pw_properties_setf(data->capture_props, PW_KEY_NODE_GROUP, "ladspa-source-%u", module->index);
 	pw_properties_setf(data->playback_props, PW_KEY_NODE_GROUP, "ladspa-source-%u", module->index);
+	pw_properties_setf(data->capture_props, "pulse.module.id", "%u", module->index);
+	pw_properties_setf(data->playback_props, "pulse.module.id", "%u", module->index);
+
+	if ((f = open_memstream(&args, &size)) == NULL)
+		return -errno;
 
-	f = open_memstream(&args, &size);
 	fprintf(f, "{");
 	pw_properties_serialize_dict(f, &module->props->dict, 0);
 	fprintf(f, " filter.graph = {");
@@ -140,12 +143,6 @@ static int module_ladspa_source_unload(struct module *module)
 	return 0;
 }
 
-static const struct module_methods module_ladspa_source_methods = {
-	VERSION_MODULE_METHODS,
-	.load = module_ladspa_source_load,
-	.unload = module_ladspa_source_unload,
-};
-
 static const struct spa_dict_item module_ladspa_source_info[] = {
 	{ PW_KEY_MODULE_AUTHOR, "Wim Taymans <wim.taymans@gmail.com>" },
 	{ PW_KEY_MODULE_DESCRIPTION, "Virtual LADSPA source" },
@@ -180,11 +177,11 @@ static void position_to_props(struct spa_audio_info_raw *info, struct pw_propert
 	pw_properties_set(props, SPA_KEY_AUDIO_POSITION, s);
 }
 
-struct module *create_module_ladspa_source(struct impl *impl, const char *argument)
+static int module_ladspa_source_prepare(struct module * const module)
 {
-	struct module *module;
-	struct module_ladspa_source_data *d;
-	struct pw_properties *props = NULL, *playback_props = NULL, *capture_props = NULL;
+	struct module_ladspa_source_data * const d = module->user_data;
+	struct pw_properties * const props = module->props;
+	struct pw_properties *playback_props = NULL, *capture_props = NULL;
 	const char *str;
 	struct spa_audio_info_raw capture_info = { 0 };
 	struct spa_audio_info_raw playback_info = { 0 };
@@ -192,22 +189,19 @@ struct module *create_module_ladspa_source(struct impl *impl, const char *argume
 
 	PW_LOG_TOPIC_INIT(mod_topic);
 
-	props = pw_properties_new_dict(&SPA_DICT_INIT_ARRAY(module_ladspa_source_info));
 	capture_props = pw_properties_new(NULL, NULL);
 	playback_props = pw_properties_new(NULL, NULL);
-	if (!props || !capture_props || !playback_props) {
+	if (!capture_props || !playback_props) {
 		res = -EINVAL;
 		goto out;
 	}
-	if (argument)
-		module_args_add_props(props, argument);
 
 	if ((str = pw_properties_get(props, "source_name")) != NULL) {
-		pw_properties_set(capture_props, PW_KEY_NODE_NAME, str);
+		pw_properties_set(playback_props, PW_KEY_NODE_NAME, str);
 		pw_properties_set(props, "source_name", NULL);
 	}
 	if ((str = pw_properties_get(props, "source_properties")) != NULL) {
-		module_args_add_props(capture_props, str);
+		module_args_add_props(playback_props, str);
 		pw_properties_set(props, "source_properties", NULL);
 	}
 	if (pw_properties_get(playback_props, PW_KEY_MEDIA_CLASS) == NULL)
@@ -215,17 +209,33 @@ struct module *create_module_ladspa_source(struct impl *impl, const char *argume
 	if (pw_properties_get(playback_props, PW_KEY_DEVICE_CLASS) == NULL)
 		pw_properties_set(playback_props, PW_KEY_DEVICE_CLASS, "filter");
 
+	if ((str = pw_properties_get(playback_props, PW_KEY_NODE_DESCRIPTION)) == NULL) {
+		str = pw_properties_get(playback_props, PW_KEY_NODE_NAME);
+		pw_properties_setf(props, PW_KEY_NODE_DESCRIPTION,
+					"%s Source", str);
+	} else {
+		pw_properties_set(props, PW_KEY_NODE_DESCRIPTION, str);
+	}
+
 	if ((str = pw_properties_get(props, "master")) != NULL ||
 	    (str = pw_properties_get(props, "source_master")) != NULL) {
-		pw_properties_set(playback_props, PW_KEY_NODE_TARGET, str);
+		if (spa_strendswith(str, ".monitor")) {
+			pw_properties_setf(capture_props, PW_KEY_TARGET_OBJECT,
+					"%.*s", (int)strlen(str)-8, str);
+			pw_properties_set(capture_props, PW_KEY_STREAM_CAPTURE_SINK,
+					"true");
+		} else {
+			pw_properties_set(capture_props, PW_KEY_TARGET_OBJECT, str);
+		}
+		pw_properties_set(props, "source_master", NULL);
 		pw_properties_set(props, "master", NULL);
 	}
 
-	if (module_args_to_audioinfo(impl, props, &capture_info) < 0) {
+	if (module_args_to_audioinfo(module->impl, props, &playback_info) < 0) {
 		res = -EINVAL;
 		goto out;
 	}
-	playback_info = capture_info;
+	capture_info = playback_info;
 
 	position_to_props(&capture_info, capture_props);
 	position_to_props(&playback_info, playback_props);
@@ -233,23 +243,23 @@ struct module *create_module_ladspa_source(struct impl *impl, const char *argume
 	if (pw_properties_get(capture_props, PW_KEY_NODE_PASSIVE) == NULL)
 		pw_properties_set(capture_props, PW_KEY_NODE_PASSIVE, "true");
 
-	module = module_new(impl, &module_ladspa_source_methods, sizeof(*d));
-	if (module == NULL) {
-		res = -errno;
-		goto out;
-	}
-
-	module->props = props;
-	d = module->user_data;
 	d->module = module;
 	d->capture_props = capture_props;
 	d->playback_props = playback_props;
 
-	return module;
+	return 0;
 out:
-	pw_properties_free(props);
 	pw_properties_free(playback_props);
 	pw_properties_free(capture_props);
-	errno = -res;
-	return NULL;
+
+	return res;
 }
+
+DEFINE_MODULE_INFO(module_ladspa_source) = {
+	.name = "module-ladspa-source",
+	.prepare = module_ladspa_source_prepare,
+	.load = module_ladspa_source_load,
+	.unload = module_ladspa_source_unload,
+	.properties = &SPA_DICT_INIT_ARRAY(module_ladspa_source_info),
+	.data_size = sizeof(struct module_ladspa_source_data),
+};
