@@ -52,7 +52,7 @@
 #include <stdio.h>
 #include <errno.h>
 #include <sys/stat.h>
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__MidnightBSD__)
 #include <sys/thr.h>
 #endif
 #include <fcntl.h>
@@ -76,12 +76,46 @@
 
 /** \page page_module_rt PipeWire Module: RT
  *
- * The `rt` module uses the operating system's scheduler to enable realtime
- * scheduling for certain threads to assist with low latency audio processing.
+ * The `rt` modules can give real-time priorities to processing threads.
+ *
+ * It uses the operating system's scheduler to enable realtime scheduling
+ * for certain threads to assist with low latency audio processing.
  * This requires `RLIMIT_RTPRIO` to be set to a value that's equal to this
  * module's `rt.prio` parameter or higher. Most distros will come with some
  * package that configures this for certain groups or users. If this is not set
  * up and DBus is available, then this module will fall back to using RTKit.
+ *
+ * ## Module Options
+ *
+ * - `nice.level`: The nice value set for the application thread. It improves
+ *                 performance of the communication with the pipewire daemon.
+ * - `rt.prio`: The realtime priority of the data thread. Higher values are
+ *              higher priority.
+ * - `rt.time.soft`, `rt.time.hard`: The amount of CPU time an RT thread can
+ *              consume without doing any blocking calls before the kernel kills
+ *              the thread. This is a safety measure to avoid lockups of the complete
+ *              system when some thread consumes 100%.
+
+ * The nice level is by default set to an invalid value so that clients don't
+ * automatically have the nice level raised.
+ *
+ * The PipeWire server processes are explicitly configured with a valid nice level.
+ *
+ * ## Example configuration
+ *
+ *\code{.unparsed}
+ * context.modules = [
+ * {   name = libpipewire-module-rt
+ *     args = {
+ *         #nice.level   = 20
+ *         #rt.prio      = 88
+ *         #rt.time.soft = -1
+ *         #rt.time.hard = -1
+ *     }
+ *     flags = [ ifexists nofail ]
+ * }
+ * ]
+ *\endcode
  */
 
 #define NAME "rt"
@@ -119,6 +153,11 @@ static const struct spa_dict_item module_props[] = {
 #ifdef HAVE_DBUS
 #define RTKIT_SERVICE_NAME "org.freedesktop.RealtimeKit1"
 #define RTKIT_OBJECT_PATH "/org/freedesktop/RealtimeKit1"
+#define RTKIT_INTERFACE "org.freedesktop.RealtimeKit1"
+
+#define XDG_PORTAL_SERVICE_NAME "org.freedesktop.portal.Desktop"
+#define XDG_PORTAL_OBJECT_PATH "/org/freedesktop/portal/desktop"
+#define XDG_PORTAL_INTERFACE "org.freedesktop.portal.Realtime"
 
 /** \cond */
 struct pw_rtkit_bus {
@@ -150,7 +189,11 @@ struct impl {
 
 #ifdef HAVE_DBUS
 	bool use_rtkit;
-	struct pw_rtkit_bus *system_bus;
+	/* For D-Bus. These are const static. */
+	const char* service_name;
+	const char* object_path;
+	const char* interface;
+	struct pw_rtkit_bus *rtkit_bus;
 
 	/* These are only for the RTKit implementation to fill in the `thread`
 	 * struct. Since there's barely any overhead here we'll do this
@@ -171,7 +214,7 @@ static pid_t _gettid(void)
 	return (pid_t) gettid();
 #elif defined(__linux__)
 	return syscall(SYS_gettid);
-#elif defined(__FreeBSD__)
+#elif defined(__FreeBSD__) || defined(__MidnightBSD__)
 	long pid;
 	thr_self(&pid);
 	return (pid_t)pid;
@@ -181,7 +224,7 @@ static pid_t _gettid(void)
 }
 
 #ifdef HAVE_DBUS
-struct pw_rtkit_bus *pw_rtkit_bus_get_system(void)
+struct pw_rtkit_bus *pw_rtkit_bus_get(DBusBusType bus_type)
 {
 	struct pw_rtkit_bus *bus;
 	DBusError error;
@@ -197,7 +240,7 @@ struct pw_rtkit_bus *pw_rtkit_bus_get_system(void)
 	if (bus == NULL)
 		return NULL;
 
-	bus->bus = dbus_bus_get_private(DBUS_BUS_SYSTEM, &error);
+	bus->bus = dbus_bus_get_private(bus_type, &error);
 	if (bus->bus == NULL)
 		goto error;
 
@@ -207,12 +250,33 @@ struct pw_rtkit_bus *pw_rtkit_bus_get_system(void)
 
 error:
 	free(bus);
-	pw_log_error("Failed to connect to system bus: %s", error.message);
+	pw_log_error("Failed to connect to %s bus: %s",
+		     bus_type == DBUS_BUS_SYSTEM ? "system" : "session", error.message);
 	dbus_error_free(&error);
 	errno = ECONNREFUSED;
 	return NULL;
 }
 
+struct pw_rtkit_bus *pw_rtkit_bus_get_system(void)
+{
+	return pw_rtkit_bus_get(DBUS_BUS_SYSTEM);
+}
+
+struct pw_rtkit_bus *pw_rtkit_bus_get_session(void)
+{
+	return pw_rtkit_bus_get(DBUS_BUS_SESSION);
+}
+
+bool pw_rtkit_check_xdg_portal(struct pw_rtkit_bus *system_bus)
+{
+	if (!dbus_bus_name_has_owner(system_bus->bus, XDG_PORTAL_SERVICE_NAME, NULL)) {
+		pw_log_warn("Can't find %s. Is xdg-desktop-portal running?", XDG_PORTAL_SERVICE_NAME);
+		return false;
+	}
+
+	return true;
+}
+
 void pw_rtkit_bus_free(struct pw_rtkit_bus *system_bus)
 {
 	dbus_connection_close(system_bus->bus);
@@ -236,7 +300,7 @@ static int translate_error(const char *name)
 	return -EIO;
 }
 
-static long long rtkit_get_int_property(struct pw_rtkit_bus *connection, const char *propname,
+static long long rtkit_get_int_property(struct impl *impl, const char *propname,
 					long long *propval)
 {
 	DBusMessage *m = NULL, *r = NULL;
@@ -246,19 +310,19 @@ static long long rtkit_get_int_property(struct pw_rtkit_bus *connection, const c
 	DBusError error;
 	int current_type;
 	long long ret;
-	const char *interfacestr = "org.freedesktop.RealtimeKit1";
+	struct pw_rtkit_bus *connection = impl->rtkit_bus;
 
 	dbus_error_init(&error);
 
-	if (!(m = dbus_message_new_method_call(RTKIT_SERVICE_NAME,
-					       RTKIT_OBJECT_PATH,
+	if (!(m = dbus_message_new_method_call(impl->service_name,
+					       impl->object_path,
 					       "org.freedesktop.DBus.Properties", "Get"))) {
 		ret = -ENOMEM;
 		goto finish;
 	}
 
 	if (!dbus_message_append_args(m,
-				      DBUS_TYPE_STRING, &interfacestr,
+				      DBUS_TYPE_STRING, &impl->interface,
 				      DBUS_TYPE_STRING, &propname, DBUS_TYPE_INVALID)) {
 		ret = -ENOMEM;
 		goto finish;
@@ -315,60 +379,63 @@ finish:
 	return ret;
 }
 
-int pw_rtkit_get_max_realtime_priority(struct pw_rtkit_bus *connection)
+int pw_rtkit_get_max_realtime_priority(struct impl *impl)
 {
 	long long retval;
 	int err;
 
-	err = rtkit_get_int_property(connection, "MaxRealtimePriority", &retval);
+	err = rtkit_get_int_property(impl, "MaxRealtimePriority", &retval);
 	return err < 0 ? err : retval;
 }
 
-int pw_rtkit_get_min_nice_level(struct pw_rtkit_bus *connection, int *min_nice_level)
+int pw_rtkit_get_min_nice_level(struct impl *impl, int *min_nice_level)
 {
 	long long retval;
 	int err;
 
-	err = rtkit_get_int_property(connection, "MinNiceLevel", &retval);
+	err = rtkit_get_int_property(impl, "MinNiceLevel", &retval);
 	if (err >= 0)
 		*min_nice_level = retval;
 	return err;
 }
 
-long long pw_rtkit_get_rttime_usec_max(struct pw_rtkit_bus *connection)
+long long pw_rtkit_get_rttime_usec_max(struct impl *impl)
 {
 	long long retval;
 	int err;
 
-	err = rtkit_get_int_property(connection, "RTTimeUSecMax", &retval);
+	err = rtkit_get_int_property(impl, "RTTimeUSecMax", &retval);
 	return err < 0 ? err : retval;
 }
 
-int pw_rtkit_make_realtime(struct pw_rtkit_bus *connection, pid_t thread, int priority)
+int pw_rtkit_make_realtime(struct impl *impl, pid_t thread, int priority)
 {
 	DBusMessage *m = NULL, *r = NULL;
+	dbus_uint64_t pid;
 	dbus_uint64_t u64;
 	dbus_uint32_t u32;
 	DBusError error;
 	int ret;
+	struct pw_rtkit_bus *connection = impl->rtkit_bus;
 
 	dbus_error_init(&error);
 
 	if (thread == 0)
 		thread = _gettid();
 
-	if (!(m = dbus_message_new_method_call(RTKIT_SERVICE_NAME,
-					       RTKIT_OBJECT_PATH,
-					       "org.freedesktop.RealtimeKit1",
-					       "MakeThreadRealtime"))) {
+	if (!(m = dbus_message_new_method_call(impl->service_name,
+					       impl->object_path, impl->interface,
+					       "MakeThreadRealtimeWithPID"))) {
 		ret = -ENOMEM;
 		goto finish;
 	}
 
+	pid = (dbus_uint64_t) getpid();
 	u64 = (dbus_uint64_t) thread;
 	u32 = (dbus_uint32_t) priority;
 
 	if (!dbus_message_append_args(m,
+				      DBUS_TYPE_UINT64, &pid,
 				      DBUS_TYPE_UINT64, &u64,
 				      DBUS_TYPE_UINT32, &u32, DBUS_TYPE_INVALID)) {
 		ret = -ENOMEM;
@@ -401,31 +468,34 @@ finish:
 	return ret;
 }
 
-int pw_rtkit_make_high_priority(struct pw_rtkit_bus *connection, pid_t thread, int nice_level)
+int pw_rtkit_make_high_priority(struct impl *impl, pid_t thread, int nice_level)
 {
 	DBusMessage *m = NULL, *r = NULL;
+	dbus_uint64_t pid;
 	dbus_uint64_t u64;
 	dbus_int32_t s32;
 	DBusError error;
 	int ret;
+	struct pw_rtkit_bus *connection = impl->rtkit_bus;
 
 	dbus_error_init(&error);
 
 	if (thread == 0)
 		thread = _gettid();
 
-	if (!(m = dbus_message_new_method_call(RTKIT_SERVICE_NAME,
-					       RTKIT_OBJECT_PATH,
-					       "org.freedesktop.RealtimeKit1",
-					       "MakeThreadHighPriority"))) {
+	if (!(m = dbus_message_new_method_call(impl->service_name,
+					       impl->object_path, impl->interface,
+					       "MakeThreadHighPriorityWithPID"))) {
 		ret = -ENOMEM;
 		goto finish;
 	}
 
+	pid = (dbus_uint64_t) getpid();
 	u64 = (dbus_uint64_t) thread;
 	s32 = (dbus_int32_t) nice_level;
 
 	if (!dbus_message_append_args(m,
+				      DBUS_TYPE_UINT64, &pid,
 				      DBUS_TYPE_UINT64, &u64,
 				      DBUS_TYPE_INT32, &s32, DBUS_TYPE_INVALID)) {
 		ret = -ENOMEM;
@@ -464,13 +534,12 @@ finish:
 static void module_destroy(void *data)
 {
 	struct impl *impl = data;
-
-	pw_thread_utils_set(NULL);
+	pw_context_set_object(impl->context, SPA_TYPE_INTERFACE_ThreadUtils, NULL);
 	spa_hook_remove(&impl->module_listener);
 
 #ifdef HAVE_DBUS
-	if (impl->system_bus)
-		pw_rtkit_bus_free(impl->system_bus);
+	if (impl->rtkit_bus)
+		pw_rtkit_bus_free(impl->rtkit_bus);
 #endif
 
 	free(impl);
@@ -487,9 +556,8 @@ static const struct pw_impl_module_events module_events = {
  */
 static bool check_realtime_privileges(rlim_t priority)
 {
-	int old_policy;
+	int err, old_policy, new_policy = REALTIME_POLICY;
 	struct sched_param old_sched_params;
-	int new_policy = REALTIME_POLICY;
 	struct sched_param new_sched_params;
 
 	/* We could check `RLIMIT_RTPRIO`, but the BSDs generally don't have
@@ -497,24 +565,23 @@ static bool check_realtime_privileges(rlim_t priority)
 	 * scheduling without that rlimit being set such as `CAP_SYS_NICE` or
 	 * running as root. Instead of checking a bunch of preconditions, we
 	 * just try if setting realtime scheduling works or not. */
-	if ((old_policy = sched_getscheduler(0)) < 0 ||
-	    sched_getparam(0, &old_sched_params) != 0) {
+	if ((err = pthread_getschedparam(pthread_self(),&old_policy,&old_sched_params)) != 0) {
+		pw_log_warn("Failed to check RLIMIT_RTPRIO: %s", strerror(err));
 		return false;
 	}
 
 	/* If the current scheduling policy has `SCHED_RESET_ON_FORK` set, then
-	 * this also needs to be set here or `sched_setscheduler()` will return
+	 * this also needs to be set here or `pthread_setschedparam()` will return
 	 * an error code. Similarly, if it is not set, then we don't want to set
 	 * it here as it would irreversible change the current thread's
 	 * scheduling policy. */
 	spa_zero(new_sched_params);
 	new_sched_params.sched_priority = priority;
-	if ((old_policy & PW_SCHED_RESET_ON_FORK) != 0) {
+	if ((old_policy & PW_SCHED_RESET_ON_FORK) != 0)
 		new_policy |= PW_SCHED_RESET_ON_FORK;
-	}
 
-	if (sched_setscheduler(0, new_policy, &new_sched_params) == 0) {
-		sched_setscheduler(0, old_policy, &old_sched_params);
+	if (pthread_setschedparam(pthread_self(), new_policy, &new_sched_params) == 0) {
+		pthread_setschedparam(pthread_self(), old_policy, &old_sched_params);
 		return true;
 	} else {
 		return false;
@@ -529,13 +596,13 @@ static int sched_set_nice(int nice_level)
 		return -errno;
 }
 
-static int set_nice(struct impl *impl, int nice_level)
+static int set_nice(struct impl *impl, int nice_level, bool warn)
 {
 	int res = 0;
 
 #ifdef HAVE_DBUS
 	if (impl->use_rtkit)
-		res = pw_rtkit_make_high_priority(impl->system_bus, 0, nice_level);
+		res = pw_rtkit_make_high_priority(impl, 0, nice_level);
 	else
 		res = sched_set_nice(nice_level);
 #else
@@ -543,13 +610,13 @@ static int set_nice(struct impl *impl, int nice_level)
 #endif
 
 	if (res < 0) {
-		pw_log_warn("could not set nice-level to %d: %s",
-				nice_level, spa_strerror(res));
+		if (warn)
+			pw_log_warn("could not set nice-level to %d: %s",
+					nice_level, spa_strerror(res));
 	} else {
 		pw_log_info("main thread nice level set to %d",
 				nice_level);
 	}
-
 	return res;
 }
 
@@ -565,11 +632,11 @@ static int set_rlimit(struct impl *impl)
 #ifdef HAVE_DBUS
 	if (impl->use_rtkit) {
 		long long rttime;
-		rttime = pw_rtkit_get_rttime_usec_max(impl->system_bus);
+		rttime = pw_rtkit_get_rttime_usec_max(impl);
 		if (rttime >= 0) {
 			if ((rlim_t)rttime < rl.rlim_cur) {
-				pw_log_debug("clamping rt.time.soft from %ld to %lld because of RTKit",
-					     rl.rlim_cur, rttime);
+				pw_log_debug("clamping rt.time.soft from %llu to %lld because of RTKit",
+					     (long long)rl.rlim_cur, rttime);
 			}
 
 			rl.rlim_cur = SPA_MIN(rl.rlim_cur, (rlim_t)rttime);
@@ -613,7 +680,7 @@ static int impl_acquire_rt_sched(struct spa_thread *thread, int priority)
 	return 0;
 }
 
-static int impl_drop_rt_generic(void *data, struct spa_thread *thread)
+static int impl_drop_rt_generic(void *object, struct spa_thread *thread)
 {
 	struct sched_param sp;
 	pthread_t pt = (pthread_t)thread;
@@ -654,12 +721,12 @@ static void *custom_start(void *data)
 	return this->start(this->arg);
 }
 
-static struct spa_thread *impl_create(void *data, const struct spa_dict *props,
+static struct spa_thread *impl_create(void *object, const struct spa_dict *props,
 		void *(*start_routine)(void*), void *arg)
 {
-	struct impl *impl = data;
+	struct impl *impl = object;
 	struct thread *this;
-	int err;
+	struct spa_thread *thread;
 
 	this = calloc(1, sizeof(*this));
 	this->impl = impl;
@@ -668,27 +735,27 @@ static struct spa_thread *impl_create(void *data, const struct spa_dict *props,
 
 	/* This thread list is only used for the RTKit implementation */
 	pthread_mutex_lock(&impl->lock);
-	err = pthread_create(&this->thread, NULL, custom_start, this);
-	if (err != 0)
+	thread = pw_thread_utils_create(props, custom_start, this);
+	if (thread == NULL)
 		goto exit;
 
+	this->thread = (pthread_t)thread;
 	pthread_cond_wait(&impl->cond, &impl->lock);
 
 	spa_list_append(&impl->threads_list, &this->link);
 exit:
 	pthread_mutex_unlock(&impl->lock);
 
-	if (err != 0) {
-		errno = err;
+	if (thread == NULL) {
 		free(this);
 		return NULL;
 	}
-	return (struct spa_thread*)this->thread;
+	return thread;
 }
 
-static int impl_join(void *data, struct spa_thread *thread, void **retval)
+static int impl_join(void *object, struct spa_thread *thread, void **retval)
 {
-	struct impl *impl = data;
+	struct impl *impl = object;
 	pthread_t pt = (pthread_t)thread;
 	struct thread *thr;
 
@@ -702,15 +769,15 @@ static int impl_join(void *data, struct spa_thread *thread, void **retval)
 	return pthread_join(pt, retval);
 }
 
-static int impl_get_rt_range(void *data, const struct spa_dict *props,
+static int impl_get_rt_range(void *object, const struct spa_dict *props,
 		int *min, int *max)
 {
-	struct impl *impl = data;
+	struct impl *impl = object;
 	if (impl->use_rtkit) {
 		if (min)
 			*min = 1;
 		if (max)
-			*max = pw_rtkit_get_max_realtime_priority(impl->system_bus);
+			*max = pw_rtkit_get_max_realtime_priority(impl);
 	} else {
 		if (min)
 			*min = sched_get_priority_min(REALTIME_POLICY);
@@ -736,9 +803,9 @@ static pid_t impl_gettid(struct impl *impl, pthread_t pt)
 	return pid;
 }
 
-static int impl_acquire_rt(void *data, struct spa_thread *thread, int priority)
+static int impl_acquire_rt(void *object, struct spa_thread *thread, int priority)
 {
-	struct impl *impl = data;
+	struct impl *impl = object;
 	struct sched_param sp;
 	int err, rtprio_limit;
 	pthread_t pt = (pthread_t)thread;
@@ -751,7 +818,7 @@ static int impl_acquire_rt(void *data, struct spa_thread *thread, int priority)
 
 	if (impl->use_rtkit) {
 		pid = impl_gettid(impl, pt);
-		rtprio_limit = pw_rtkit_get_max_realtime_priority(impl->system_bus);
+		rtprio_limit = pw_rtkit_get_max_realtime_priority(impl);
 		if (rtprio_limit >= 0 && rtprio_limit < priority) {
 			pw_log_info("dropping requested priority %d for thread %d down to %d because of RTKit limits", priority, pid, rtprio_limit);
 			priority = rtprio_limit;
@@ -764,7 +831,7 @@ static int impl_acquire_rt(void *data, struct spa_thread *thread, int priority)
 			pw_log_debug("SCHED_OTHER|SCHED_RESET_ON_FORK worked.");
 		}
 
-		if ((err = pw_rtkit_make_realtime(impl->system_bus, pid, priority)) < 0) {
+		if ((err = pw_rtkit_make_realtime(impl, pid, priority)) < 0) {
 			pw_log_warn("could not make thread %d realtime using RTKit: %s", pid, spa_strerror(err));
 			return err;
 		}
@@ -787,26 +854,18 @@ static const struct spa_thread_utils_methods impl_thread_utils = {
 
 #else /* HAVE_DBUS */
 
-static struct spa_thread *impl_create(void *data, const struct spa_dict *props,
+static struct spa_thread *impl_create(void *object, const struct spa_dict *props,
 		void *(*start_routine)(void*), void *arg)
 {
-	pthread_t pt;
-	int err;
-
-	err = pthread_create(&pt, NULL, start_routine, arg);
-	if (err != 0) {
-		errno = err;
-		return NULL;
-	}
-	return (struct spa_thread*)pt;
+	return pw_thread_utils_create(props, start_routine, arg);
 }
 
-static int impl_join(void *data, struct spa_thread *thread, void **retval)
+static int impl_join(void *object, struct spa_thread *thread, void **retval)
 {
-	return pthread_join((pthread_t)thread, retval);
+	return pw_thread_utils_join(thread, retval);
 }
 
-static int impl_get_rt_range(void *data, const struct spa_dict *props,
+static int impl_get_rt_range(void *object, const struct spa_dict *props,
 		int *min, int *max)
 {
 	if (min)
@@ -816,9 +875,9 @@ static int impl_get_rt_range(void *data, const struct spa_dict *props,
 	return 0;
 }
 
-static int impl_acquire_rt(void *data, struct spa_thread *thread, int priority)
+static int impl_acquire_rt(void *object, struct spa_thread *thread, int priority)
 {
-	struct impl *impl = data;
+	struct impl *impl = object;
 
 	/* See the docstring on `spa_thread_utils_methods::acquire_rt` */
 	if (priority == -1) {
@@ -840,31 +899,17 @@ static const struct spa_thread_utils_methods impl_thread_utils = {
 
 
 #ifdef HAVE_DBUS
-static int should_use_rtkit(struct impl *impl, struct pw_context *context, bool *use_rtkit)
+static int check_rtkit(struct impl *impl, struct pw_context *context, bool *can_use_rtkit)
 {
 	const struct pw_properties *context_props;
 	const char *str;
 
-	*use_rtkit = true;
+	*can_use_rtkit = true;
 
 	if ((context_props = pw_context_get_properties(context)) != NULL &&
 	    (str = pw_properties_get(context_props, "support.dbus")) != NULL &&
 	    !pw_properties_parse_bool(str))
-		*use_rtkit = false;
-
-	/* If the user has permissions to use regular realtime scheduling, then
-	 * we'll use that instead of RTKit */
-	if (check_realtime_privileges(impl->rt_prio)) {
-		*use_rtkit = false;
-	} else {
-		if (!(*use_rtkit)) {
-			pw_log_warn("neither regular realtime scheduling nor RTKit are available");
-			return -ENOTSUP;
-		}
-
-		/* TODO: Should this be pw_log_warn or pw_log_debug instead? */
-		pw_log_info("could not use realtime scheduling, falling back to using RTKit instead");
-	}
+		*can_use_rtkit = false;
 
 	return 0;
 }
@@ -898,40 +943,75 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	impl->rt_time_soft = pw_properties_get_int32(props, "rt.time.soft", DEFAULT_RT_TIME_SOFT);
 	impl->rt_time_hard = pw_properties_get_int32(props, "rt.time.hard", DEFAULT_RT_TIME_HARD);
 
+	bool can_use_rtkit = false, use_rtkit = false;
+
 #ifdef HAVE_DBUS
 	spa_list_init(&impl->threads_list);
 	pthread_mutex_init(&impl->lock, NULL);
 	pthread_cond_init(&impl->cond, NULL);
 
-	if ((res = should_use_rtkit(impl, context, &impl->use_rtkit)) < 0) {
+	if ((res = check_rtkit(impl, context, &can_use_rtkit)) < 0)
 		goto error;
-	}
-
-	if (impl->use_rtkit) {
-		impl->system_bus = pw_rtkit_bus_get_system();
-		if (impl->system_bus == NULL) {
-			res = -errno;
-			pw_log_warn("could not get system bus: %m");
+#endif
+	/* If the user has permissions to use regular realtime scheduling, as well as
+	 * the nice level we want, then we'll use that instead of RTKit */
+	if (!check_realtime_privileges(impl->rt_prio)) {
+		if (!can_use_rtkit) {
+			res = -ENOTSUP;
+			pw_log_warn("regular realtime scheduling not available (RTKit fallback disabled)");
 			goto error;
 		}
+		use_rtkit = true;
 	}
-#else
-	if (!check_realtime_privileges(impl->rt_prio)) {
-		res = -ENOTSUP;
-		pw_log_warn("regular realtime scheduling not available (RTKit fallback disabled)");
-		goto error;
+
+	if (IS_VALID_NICE_LEVEL(impl->nice_level)) {
+		if (set_nice(impl, impl->nice_level, !can_use_rtkit) < 0)
+			use_rtkit = can_use_rtkit;
+	}
+
+#ifdef HAVE_DBUS
+	impl->use_rtkit = use_rtkit;
+	if (impl->use_rtkit) {
+		/* Checking xdg-desktop-portal. It works fine in all situations. */
+		impl->rtkit_bus = pw_rtkit_bus_get_session();
+		if (impl->rtkit_bus != NULL) {
+			if (pw_rtkit_check_xdg_portal(impl->rtkit_bus)) {
+				impl->service_name = XDG_PORTAL_SERVICE_NAME;
+				impl->object_path = XDG_PORTAL_OBJECT_PATH;
+				impl->interface = XDG_PORTAL_INTERFACE;
+			} else {
+				pw_log_warn("found session bus but no portal");
+				pw_rtkit_bus_free(impl->rtkit_bus);
+				impl->rtkit_bus = NULL;
+			}
+		}
+		/* Failed to get xdg-desktop-portal, try to use rtkit. */
+		if (impl->rtkit_bus == NULL) {
+			impl->rtkit_bus = pw_rtkit_bus_get_system();
+			if (impl->rtkit_bus != NULL) {
+				impl->service_name = RTKIT_SERVICE_NAME;
+				impl->object_path = RTKIT_OBJECT_PATH;
+				impl->interface = RTKIT_INTERFACE;
+			} else {
+				res = -errno;
+				pw_log_warn("could not get system bus: %m");
+				goto error;
+			}
+		}
+		/* Retry set_nice with rtkit */
+		if (IS_VALID_NICE_LEVEL(impl->nice_level))
+			set_nice(impl, impl->nice_level, true);
 	}
 #endif
-
-	if (IS_VALID_NICE_LEVEL(impl->nice_level))
-		set_nice(impl, impl->nice_level);
 	set_rlimit(impl);
 
 	impl->thread_utils.iface = SPA_INTERFACE_INIT(
 			SPA_TYPE_INTERFACE_ThreadUtils,
 			SPA_VERSION_THREAD_UTILS,
 			&impl_thread_utils, impl);
-	pw_thread_utils_set(&impl->thread_utils);
+
+	pw_context_set_object(context, SPA_TYPE_INTERFACE_ThreadUtils,
+			&impl->thread_utils);
 
 	pw_impl_module_add_listener(module, &impl->module_listener, &module_events, impl);
 
@@ -952,8 +1032,8 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 
 error:
 #ifdef HAVE_DBUS
-	if (impl->system_bus)
-		pw_rtkit_bus_free(impl->system_bus);
+	if (impl->rtkit_bus)
+		pw_rtkit_bus_free(impl->rtkit_bus);
 #endif
 	free(impl);
 done:
